#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt import QtGui
from .qt_control import QtControl

from ...components.image_view import AbstractTkImageView


class QtImageView(QtControl, AbstractTkImageView):
    """ A Qt4 implementation of ImageView.

    """
    #: The internal cached size hint which is used to determine whether
    #: of not a size hint updated event should be emitted when the image
    #: in the QLabel changes.
    _cached_size_hint = None

    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying QLabel control.

        """
        self.widget = QtGui.QLabel(parent)

    def initialize(self):
        """ Initializes the attributes on the underlying control.

        """
        super(QtImageView, self).initialize()
        shell = self.shell_obj
        self.set_image(shell.image)
        self.set_scale_to_fit(shell.scale_to_fit)

    #--------------------------------------------------------------------------
    # Implementation
    #--------------------------------------------------------------------------
    def shell_image_changed(self, image):
        """ The change handler for the 'image' attribute on the shell 
        component.

        """
        self.set_image(image)
    
    def shell_scale_to_fit_changed(self, scale_to_fit):
        """ The change handler for the 'scale_to_fit' attribute on the 
        shell component.

        """
        self.set_scale_to_fit(scale_to_fit)

    #--------------------------------------------------------------------------
    # Widget Update Methods
    #--------------------------------------------------------------------------
    def set_image(self, image):
        """ Sets the image on the underlying QLabel.

        """
        self.widget.setPixmap(image.as_QPixmap())

        # Emit a size hint updated event if the size hint has actually
        # changed. This is an optimization so that a constraints update
        # only occurs when the size hint has actually changed. This 
        # logic must be implemented here so that the label has been
        # updated before the new size hint is computed. Placing this
        # logic on the shell object would not guarantee that the label
        # has been updated at the time the change handler is called.
        cached = self._cached_size_hint
        hint = self._cached_size_hint = self.size_hint()
        if cached != hint:
            self.shell_obj.size_hint_updated()
    
    def set_scale_to_fit(self, scale_to_fit):        
        """ Sets whether or not the image scales with the underlying 
        control.

        """
        self.widget.setScaledContents(scale_to_fit)
        
        # See the comment in set_image(...) about the size hint update
        # notification. The same logic applies here.
        cached = self._cached_size_hint
        hint = self._cached_size_hint = self.size_hint()
        if cached != hint:
            self.shell_obj.size_hint_updated()




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .enaml_test_case import EnamlTestCase, required_method


class TestComboBox(EnamlTestCase):
    """ Logic for testing combo boxes.

    Tooklit testcases need to provide the following methods

    Abstract Methods
    ----------------
    get_selected_text(self, widget)
        Get the current selected text of a combo box.

    get_item_text(self, widget, index)
        Get the text of a combo box item at a particular index.

    select_item(self, widget, index)
        Fire an event to simulate the selection of an item.

    """

    def setUp(self):
        """ Setup before the combo box tests.

        """

        enaml = """
enamldef MainView(MainWindow):
    attr events
    ComboBox:
        name = 'cmb'
        items = [int, float, oct]
        value = float
        to_string = lambda x: str(x) + '!' if x is not None else ''
        selected :: events.append(('selected', event.new))
"""

        self.events = []
        self.view = self.parse_and_create(enaml, events=self.events)
        self.component = self.component_by_name(self.view, 'cmb')
        self.widget = self.component.toolkit_widget

    def test_initialization(self):
        """ Test the initial state.

        """
        component = self.component
        str_value = component.selected_text
        self.assertEqual(self.get_selected_text(self.widget), str_value)
        self.assertEqual(component.value, component.items[1])
        self.assertEqual(self.events, [])

    def test_items(self):
        """ Check that the Enaml combo box items match the toolkit widget.

        """
        component = self.component
        for i, item in enumerate(component.items):
            widget_text = self.get_item_text(self.widget, i)
            self.assertEqual(widget_text, component.to_string(item))

    def test_to_string(self):
        """ Update the ComboBox.to_string callable.

        """
        component = self.component
        component.to_string = lambda x: str(x) + '?'
        self.test_items()

    def test_selected_event(self):
        """ Fire an event for item selection.

        """
        self.select_item(self.widget, 2)
        self.assertEqual(self.events, [('selected', oct)])

    def test_change_selected_item(self):
        """ Update the visible item when a new one is selected internally.

        """
        component = self.component
        self.assertEqual(component.value, float)
        index = 2
        self.select_item(self.widget, index)
        self.assertEqual(component.value, oct)
        self.assertEqual(self.events, [('selected', oct)])

    def test_append_item(self):
        """ Add an item on the Enaml side; see if the toolkit widget 
        updates.

        """
        component = self.component
        component.items.append('hello')
        self.test_items()

    def test_remove_item(self):
        """ Remove an item on the Enaml side; see if the toolkit widget 
        updates.

        """
        component = self.component
        component.items.pop(0)
        self.test_items()

    def test_deselect(self):
        """ Assert that an invalid value sets the index to -1.

        """
        component = self.component
        component.value = hex
        self.assertTrue(component.value is hex)
        self.assertEqual(component.index, -1)

    def test_value_when_items_change(self):
        """ Assert that the selection moves correctly when the items 
        change.

        """
        component = self.component
        component.value = int
        self.assertEqual(component.value, int)
        self.assertEqual(component.index, 0)
        component.items.insert(0, hex)
        self.assertEqual(component.index, 1)
        self.assertEqual(self.events, [])

    def test_index_when_items_change(self):
        """ Assert that the index is -1 when the value is removed from 
        the items list.

        """
        component = self.component
        component.value = int
        component.items.pop(0)
        self.assertEqual(component.index, -1)
        self.assertEqual(self.events, [])

    #--------------------------------------------------------------------------
    # Abstract methods
    #--------------------------------------------------------------------------
    @required_method
    def get_selected_text(self, widget):
        """ Get the current selected text of a combo box.

        """
        pass

    @required_method
    def get_item_text(self, widget, index):
        """ Get the text of a combo box item at a particular index.

        """
        pass

    @required_method
    def select_item(self, widget, index):
        """ Fire an event to simulate the selection of an item.

        """
        pass




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt import QtGui
from .qt_abstract_item_view import QtAbstractItemView

from ...components.table_view import AbstractTkTableView


class QtTableView(QtAbstractItemView, AbstractTkTableView):
    """ A Qt implementation of TableView.

    """
    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Create the underlying QTableView control.

        """
        self.widget = QtGui.QTableView(parent)

    def initialize(self):
        """ Initialize the widget with the attributes of this instance.

        """
        super(QtTableView, self).initialize()
        shell = self.shell_obj
        self.set_vertical_header_visible(shell.vertical_header_visible)
        self.set_horizontal_header_visible(shell.horizontal_header_visible)

    #--------------------------------------------------------------------------
    # Implementation
    #-------------------------------------------------------------------------- 
    def shell_vertical_header_visible_changed(self, visible):
        """ The change handler for the 'vertical_header_visible' 
        attribute of the shell object.

        """
        self.set_vertical_header_visible(visible)
    
    def shell_horizontal_header_visible_changed(self, visible):
        """ The change handler for the 'horizontal_header_visible'
        attribute of the shell object.

        """
        self.set_horizontal_header_visible(visible)

    #--------------------------------------------------------------------------
    # Widget Update Methods
    #--------------------------------------------------------------------------
    def set_vertical_header_visible(self, visible):
        """ Sets the vertical header visibility of the widget.

        """
        if visible:
            self.widget.verticalHeader().show()
        else:
            self.widget.verticalHeader().hide()
    
    def set_horizontal_header_visible(self, visible):
        """ Sets the horizontal header visibility of the widget.

        """
        if visible:
            self.widget.horizontalHeader().show()
        else:
            self.widget.horizontalHeader().hide()




# -*- coding: UTF-8 -*-
#------------------------------------------------------------------------------
#  file: line_functions.py
#  License: LICENSE.TXT
#  Author: Ioannis Tziakos
#
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
#!/usr/bin/env python
import re

#------------------------------------------------------------------------------
#  Precompiled regexes
#------------------------------------------------------------------------------
indent_regex = re.compile(r'\s+')

#------------------------------------------------------------------------------
#  Functions to manage indention
#------------------------------------------------------------------------------

def add_indent(lines, indent=4):
    """ Add spaces to indent a list of lines.

    Arguments
    ---------
    lines : list
        The list of strings to indent.

    indent : int
        The number of spaces to add.

    Returns
    -------
    lines : list
        The indented strings (lines).

    .. note:: Empty strings are not changed

    """
    indent_str = ' ' * indent
    output = []
    for line in lines:
        if is_empty(line):
            output.append(line)
        else:
            output.append(indent_str + line)
    return output

def remove_indent(lines):
    """ Remove all indentation from the lines.

    """
    return [line.lstrip() for line in lines]

def get_indent(line):
    """ Return the indent portion of the line.

    """
    indent = indent_regex.match(line)
    if indent is None:
        return ''
    else:
        return indent.group()

#------------------------------------------------------------------------------
#  Functions to detect line type
#------------------------------------------------------------------------------

def is_empty(line):
    return not line.strip()

#------------------------------------------------------------------------------
#  Functions to adjust strings
#------------------------------------------------------------------------------

def fix_star(word):
    return word.replace('*','\*')

def fix_backspace(word):
    return word.replace('\\', '\\\\')

def replace_at(word, line, index):
    """ Replace the text in-line.

    The text in line is replaced (not inserted) with the word. The
    replacement starts at the provided index. The result is cliped to
    the input length

    Arguments
    ---------
    word : str
        The text to copy into the line.

    line : str
        The line where the copy takes place.

    index : int
        The index to start coping.

    Returns
    -------
    result : str
        line of text with the text replaced.

    """
    word_length = len(word)
    result = line[:index] + word + line[(index + word_length):]
    return result[:len(line)]





#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt.QtGui import QSpinBox, QValidator
from .qt_control import QtControl

from ...components.spin_box import AbstractTkSpinBox


class EnamlQSpinBox(QSpinBox):
    """ A QSpinBox sublcass with hooks for user supplied validation.

    """
    #: The internal storage for the Enaml validator object. 
    _validator = None

    def validator(self):
        """ Returns the Enaml validator assigned to this spin box.

        """
        return self._validator
    
    def setValidator(self, validator):
        """ Set the validator for this spin box.

        """
        self._validator = validator
        self.interpretText()

    def textFromValue(self, value):
        """ Converts the given integer to a string for display using the 
        user supplied validator object. 

        If the conversion fails due to the converter raising a ValueError
        then simple unicode(...) conversion is used.

        """
        try:
            text = self._validator.format(value)
        except ValueError:
            text = unicode(value)
        return text

    def valueFromText(self, text):
        """ Converts the user typed string into an integer for the
        control using the user supplied validator.

        """
        return self._validator.convert(text)

    def validate(self, text, pos):
        """ Validates whether or not the given text can be converted
        to an integer.

        """
        v = self._validator
        rv = v.validate(text)
        if rv == v.ACCEPTABLE:
            res = QValidator.Acceptable
        elif rv == v.INTERMEDIATE:
            res = QValidator.Intermediate
        elif rv == v.INVALID:
            res = QValidator.Invalid
        else:
            # This should never happen
            raise ValueError('Invalid validation result')
        return (res, text, pos)


class QtSpinBox(QtControl, AbstractTkSpinBox):
    """ A Qt implementation of SpinBox.

    """
    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying custom spin control.

        """
        self.widget = EnamlQSpinBox(parent)

    def initialize(self):
        """ Intializes the widget with the attributes of this instance.

        """
        super(QtSpinBox, self).initialize()
        shell = self.shell_obj
        self.set_validator(shell.validator)
        self.set_spin_low(shell.low)
        self.set_spin_high(shell.high)
        self.set_spin_step(shell.step)
        self.set_spin_value(shell.value)
        self.set_spin_wrap(shell.wrap)
        self.set_spin_tracking(shell.tracking)

    def bind(self):
        """ Binds the event handlers for the spin control.

        """
        super(QtSpinBox, self).bind()
        self.widget.valueChanged.connect(self.on_value_changed)

    #--------------------------------------------------------------------------
    # Implementation
    #--------------------------------------------------------------------------
    def shell_value_changed(self, value):
        """ The change handler for the 'value' attribute of the shell
        object.

        """
        self.set_spin_value(value)

    def shell_low_changed(self, low):
        """ The change handler for the 'low' attribute of the shell
        object.

        """
        self.set_spin_low(low)

    def shell_high_changed(self, high):
        """ The change handler for the 'high' attribute of the shell
        object.

        """
        self.set_spin_high(high)

    def shell_step_changed(self, step):
        """ The change handler for the 'step' attribute of the shell
        object.

        """
        self.set_spin_step(step)

    def shell_validator_changed(self, validator):
        """ The change handler for the 'validator' attribute of the shell
        object.

        """
        self.set_validator(validator)

    def shell_wrap_changed(self, wrap):
        """ The change handler for the 'wrap' attribute of the shell 
        object.

        """
        self.set_spin_wrap(wrap)

    def shell_tracking_changed(self, tracking):
        """ The change handler for the 'tracking' attribute of the shell
        object.

        """
        self.set_spin_tracking(tracking)

    #--------------------------------------------------------------------------
    # Event Handlers
    #--------------------------------------------------------------------------
    def on_value_changed(self):
        """ The event handler for the widget's spin event.

        """
        self.shell_obj.value = self.widget.value()

    #--------------------------------------------------------------------------
    # Widget update methods 
    #--------------------------------------------------------------------------
    def set_spin_value(self, value):
        """ Sets the value of the widget.

        """
        self.widget.setValue(value)

    def set_spin_low(self, low):
        """ Sets the minimum value of the widget.

        """
        self.widget.setMinimum(low)

    def set_spin_high(self, high):
        """ Sets the maximum value of the widget.

        """
        self.widget.setMaximum(high)

    def set_spin_step(self, step):
        """ Sets the step size of the widget.

        """
        self.widget.setSingleStep(step)

    def set_validator(self, validator):
        """ Sets the validator for the widget.

        """
        self.widget.setValidator(validator)

    def set_spin_wrap(self, wrap):
        """ Sets the wrap mode of the widget.

        """
        self.widget.setWrapping(wrap)

    def set_spin_tracking(self, tracking):
        """ Sets the keyboard tracking of the widget.

        """
        self.widget.setKeyboardTracking(tracking)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import (
    Property, Int, Instance, List, cached_property, on_trait_change,
)

from .constraints_widget import (
    ConstraintsWidget, AbstractTkConstraintsWidget,
)
from .layout_task_handler import LayoutTaskHandler
from .tab import Tab

from ..enums import TabPosition


class AbstractTkTabGroup(AbstractTkConstraintsWidget):
    """ The abstract toolkit TabGroup interface.

    """
    @abstractmethod
    def shell_tabs_changed(self, tabs):
        """ The change handler for the 'tabs' attribute of the shell 
        object.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_tab_position_changed(self, tab_position):
        """ The change handler for the 'tab_position' attribute on the 
        shell object.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_selected_index_changed(self, index):
        """ The change handler for the 'selected_index' attribute on
        the shell object.

        """
        raise NotImplementedError


class TabGroup(LayoutTaskHandler, ConstraintsWidget):
    """ A LayoutComponent that arranges its children as a group of tabs.

    The TabGroup provides a very simple way of laying out a number of 
    children as tabs, suitable for configuration dialogs and the like. 
    It is not suitable for rich tab interfaces where the group of tabs
    is the central focus of the application. For that, use a Notebook.

    """
    #: A read-only cached property that returns the tab children
    #: of this tab group.
    tabs = Property(List(Instance(Tab)), depends_on='children')

    #: A readonly property which returns the selected index. If there 
    #: are no tabs in the group, this will return -1.
    selected_index = Property(Int, depends_on='_selected_index')

    #: A readonly property which returns the selected tab. If there are 
    #: no tabs in the group, this will return None.
    selected_tab = Property(Instance(Tab), depends_on='_selected_index')

    #: A TabPosition enum value that indicate where the tabs should
    #: be drawn on the control. One of 'top', 'bottom', 'left', 'right'.
    #: The default value is 'top'.
    tab_position = TabPosition('top')

    #: How strongly a component hugs it's contents' width. A TabGroup
    #: ignores its width hug by default, so it expands freely in width.
    hug_width = 'ignore'

    #: How strongly a component hugs it's contents' height. A TabGroup
    #: ignores its height hug by default, so it expands freely in height.
    hug_height = 'ignore'

    #: The private storage for the selected tab index. This attribute
    #: should be used by the toolkit implementations to communicate the 
    #: current selection. A -1 indicates no selection.
    _selected_index = Int(-1)

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkTabGroup)
    
    #--------------------------------------------------------------------------
    # Property Getters
    #--------------------------------------------------------------------------
    @cached_property
    def _get_tabs(self):
        """ The property getter for the 'tabs' attribute.

        """
        flt = lambda child: isinstance(child, Tab)
        return filter(flt, self.children)
    
    def _get_selected_tab(self):
        """ The property getter for the 'selected_tab' attribute.

        """
        idx = self._selected_index
        tabs = self.tabs
        n = len(tabs)
        if 0 <= idx < n:
            res = tabs[idx]
        elif idx == -1 and n > 0:
            res = tabs[0]
        else:
            res = None
        return res
    
    def _get_selected_index(self):
        """ The property getter for the 'selected_index' attribute.

        """
        idx = self._selected_index
        if idx == -1 and len(self.tabs) > 0:
            idx = 0
        return idx

    #--------------------------------------------------------------------------
    # Change Handlers
    #--------------------------------------------------------------------------
    @on_trait_change('tabs, tab_position')
    def _on_tab_group_deps_changed(self):
        """ A change handler for triggering a relayout when the tabs or
        the position of the tabs change, provided that the component
        is initialized.

        """
        if self.initialized:
            self.request_relayout()

    #--------------------------------------------------------------------------
    # Overrides
    #--------------------------------------------------------------------------
    def do_relayout(self):
        """ A reimplemented LayoutTaskHandler handler method which will
        perform necessary update activity when a relayout is requested.

        """
        self.size_hint_updated()

    #--------------------------------------------------------------------------
    # Auxiliary Methods 
    #--------------------------------------------------------------------------
    def set_selected_tab(self, tab):
        """ Set the selected tab of the group to the given tab.

        """
        try:
            idx = self.tabs.index(tab)
        except ValueError:
            msg = ('Cannot set selected tab. Tab %s does not exist in '
                   'the TabGroup')
            raise ValueError(msg % tab)
        self._selected_index = idx
    
    def set_selected_index(self, idx):
        """ Set the selected index of the group to the given index.

        """
        if idx < 0 or idx >= len(self.tabs):
            msg = "Tab index %s out of range for TabGroup"
            raise IndexError(msg % idx)
        self._selected_index = idx




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import ast
import itertools
import types

from .byteplay import (
    Code, LOAD_FAST, CALL_FUNCTION, LOAD_GLOBAL, STORE_FAST, LOAD_CONST,
    LOAD_ATTR, STORE_SUBSCR, RETURN_VALUE, POP_TOP, MAKE_FUNCTION,
    STORE_NAME, LOAD_NAME, SetLineno,
)


# Increment this number whenever the compiler changes the code which it
# generates. This number is used by the import hooks to know which version
# of a .enamlc file is valid for the Enaml compiler version in use. If 
# this number is not incremented on change, it may result in .enamlc
# files which fail on import.
#
# Version History
# ---------------
# 1 : Initial compiler version - 2 February 2012
# 2 : Update line number handling - 26 March 2012
#     When compiling code objects with mode='eval', Python ignores the 
#     line number specified by the ast. The workaround is to compile the 
#     code object, then make a new copy of it with the proper firstlineno 
#     set via the types.CodeType constructor.
COMPILER_VERSION = 2


#------------------------------------------------------------------------------
# Compiler Helpers
#------------------------------------------------------------------------------
# Code that will be executed at the top of every enaml module
STARTUP = ['from enaml.core.factory import EnamlDeclaration']


# Cleanup code that will be included in every compiled enaml module
CLEANUP = ['del EnamlDeclaration']


def _var_name_generator():
    """ Returns a generator that generates sequential variable names for
    use in a code block.

    """
    count = itertools.count()
    while True:
        yield '_var_' + str(count.next())


def update_firstlineno(code, firstlineno):
    """ Returns a new code object with an updated first line number.

    """
    return types.CodeType(
        code.co_argcount, code.co_nlocals, code.co_stacksize, code.co_flags,
        code.co_code, code.co_consts, code.co_names, code.co_varnames,
        code.co_filename, code.co_name, firstlineno, code.co_lnotab, 
        code.co_freevars, code.co_cellvars,
    )


#------------------------------------------------------------------------------
# Node Visitor
#------------------------------------------------------------------------------
class _NodeVisitor(object):
    """ A node visitor class that is used as base class for the various
    Enaml compilers.

    """
    def visit(self, node):
        """  The main visitor dispatch method.

        """
        name = 'visit_%s' % node.__class__.__name__
        try:
            method = getattr(self, name)
        except AttributeError:
            method = self.default_visit
        method(node)

    def default_visit(self, node):
        """ The default visitor method. Raises an error since there 
        should not be any unhandled nodes.

        """
        raise ValueError('Unhandled Node %s.' % node)


#------------------------------------------------------------------------------
# Declaration Compiler
#------------------------------------------------------------------------------
class DeclarationCompiler(_NodeVisitor):
    """ A visitor which compiles a Declaration node into a code object.

    """
    @classmethod
    def compile(cls, node, filename):
        """ Compiles the given Declaration node into a byteplay code 
        object.

        Given this sample declaration in Enaml::
          
        FooWindow(Window):
            id: foo
            a = '12'
            PushButton:
                id: btn
                text = 'clickme'
        
        We generate bytecode that would correspond to a Python function that
        looks similar to this::
        
        def FooWindow(identifiers, toolkit):
            f_globals = globals()
            eval_ = eval
            foo_cls = eval('Window', toolkit, f_globals)
            foo = foo_cls.__enaml_call__(identifiers, toolkit)
            identifiers['foo'] = foo
            op = eval_('__operator_Equal__', toolkit, f_globals)
            op(foo, 'a', <ast>, <code>, identifiers, f_globals, toolkit)
            btn_cls = eval_('PushButton', toolkit, f_globals)
            btn = btn_cls.__enaml_call__(None, toolkit)
            identifiers['btn'] = button
            op = eval_('__operator_Equal__', toolkit, f_globals)
            op(item, 'text', <ast>, <code>, identifiers, f_globals, toolkit)
            foo.add_subcomponent(button)
            return foo
        
        """
        compiler = cls(filename)
        compiler.visit(node)
        code_ops = compiler.code_ops
        code = Code(
            code_ops, [], ['identifiers', 'toolkit'], False, False, True, 
            node.name, filename, node.lineno, node.doc,
        )
        return code

    def __init__(self, filename):
        self.filename = filename
        self.code_ops = []
        self.extend_ops = self.code_ops.extend
        self.name_gen = _var_name_generator()
        self.name_stack = []
        self.push_name = self.name_stack.append
        self.pop_name = self.name_stack.pop

    def curr_name(self):
        """ Returns the current variable name on the stack.

        """
        return self.name_stack[-1]

    def visit_Declaration(self, node):
        """ Creates the bytecode ops for a declaration node. This visitor
        handles creating the component instance and storing it's identifer
        if one is given.

        """
        name = self.name_gen.next()
        extend_ops = self.extend_ops
        self.push_name(name)
        base_code = compile(node.base.py_ast, self.filename, mode='eval')
        extend_ops([
            # f_globals = globals()
            (LOAD_GLOBAL, 'globals'),
            (CALL_FUNCTION, 0x0000),
            (STORE_FAST, 'f_globals'),

            # eval_ = eval
            (LOAD_GLOBAL, 'eval'),
            (STORE_FAST, 'eval_'),

            # foo_cls = eval('Window', toolkit, f_globals)
            # foo = foo_cls.__enaml_call__(identifiers, toolkit)
            (LOAD_FAST, 'eval_'),
            (LOAD_CONST, base_code),
            (LOAD_FAST, 'toolkit'),
            (LOAD_FAST, 'f_globals'),
            (CALL_FUNCTION, 0x0003),
            (LOAD_ATTR, '__enaml_call__'),
            (LOAD_FAST, 'identifiers'),
            (LOAD_FAST, 'toolkit'),
            (CALL_FUNCTION, 0x0002),
            (STORE_FAST, name),
        ])

        if node.identifier:
            extend_ops([
                # identifiers['foo'] = foo
                (LOAD_FAST, name),
                (LOAD_FAST, 'identifiers'),
                (LOAD_CONST, node.identifier),
                (STORE_SUBSCR, None),
            ])
        
        visit = self.visit
        for item in node.body:
            visit(item)
        
        extend_ops([
            # return foo
            (LOAD_FAST, name),
            (RETURN_VALUE, None),
        ])

        self.pop_name()

    def visit_AttributeDeclaration(self, node):
        """ Creates the bytecode ops for an attribute declaration. This
        visitor handles adding a new attribute to a component.

        """
        extend_ops = self.extend_ops

        # Load the method that's going to be called and the
        # name of the attribute being declared.
        extend_ops([
            (LOAD_FAST, self.curr_name()),
            (LOAD_ATTR, 'add_attribute'),
            (LOAD_CONST, node.name),
        ])

        # Generate the ops to the load the type (if one was given),
        # and the call the add_attribute method
        node_type = node.type
        if node_type is not None:
            type_code = compile(node_type.py_ast, self.filename, mode='eval')
            extend_ops([
                (LOAD_FAST, 'eval_'),
                (LOAD_CONST, type_code),
                (LOAD_FAST, 'toolkit'),
                (LOAD_FAST, 'f_globals'),
                (CALL_FUNCTION, 0x0003),
                (LOAD_CONST, node.is_event),
                (CALL_FUNCTION, 0x0003),
                (POP_TOP, None),
            ])
        else:
            extend_ops([
                (LOAD_CONST, 'is_event'),
                (LOAD_CONST, node.is_event),
                (CALL_FUNCTION, 0x0101),
                (POP_TOP, None),
            ])

        # Visit the default attribute binding if one exists.
        default = node.default
        if default is not None:
            self.visit(node.default)

    def visit_AttributeBinding(self, node):
        """ Creates the bytecode ops for an attribute binding. This
        visitor handles loading and calling the appropriate operator.

        """
        # A binding is accomplished by loading the appropriate binding
        # operator function and passing it the a number of arguments:
        #
        # op = eval('__operator_Equal__', toolkit, f_globals)
        # op(item, 'a', code, identifiers, f_globals, toolkit)
        fn = self.filename
        op_code = compile(node.binding.op, fn, mode='eval')
        py_ast = node.binding.expr.py_ast
        if isinstance(py_ast, ast.Module):
            expr_code = compile(py_ast, fn, mode='exec')
        else:
            # When compiling in 'eval' mode, the line number in the ast
            # gets ignored. We need to make new code object from this
            # on with the proper starting line number so that 
            # exceptions are properly reported.
            expr_code = compile(py_ast, fn, mode='eval')
            expr_code = update_firstlineno(expr_code, py_ast.lineno)
        self.extend_ops([
            (LOAD_FAST, 'eval_'),
            (LOAD_CONST, op_code),
            (LOAD_FAST, 'toolkit'),
            (LOAD_FAST, 'f_globals'),
            (CALL_FUNCTION, 0x0003),
            (LOAD_FAST, self.curr_name()),
            (LOAD_CONST, node.name),
            (LOAD_CONST, expr_code),
            (LOAD_FAST, 'identifiers'),
            (LOAD_FAST, 'f_globals'),
            (LOAD_FAST, 'toolkit'),
            (CALL_FUNCTION, 0x0006),
            (POP_TOP, None),
        ])

    def visit_Instantiation(self, node):
        """ Create the bytecode ops for a component instantiation. This 
        visitor handles calling another derived component and storing
        its identifier, if given.
        
        """
        extend_ops = self.extend_ops
        name = self.name_gen.next()
        self.push_name(name)
        op_code = compile(node.name, self.filename, mode='eval')
        extend_ops([
            # btn_cls = eval('PushButton', toolkit, f_globals)
            # btn = btn_cls.__enaml_call__(None, toolkit)
            # When instantiating a Declaration, it is called without
            # identifiers, so that it creates it's own new identifiers
            # scope. This means that derived declarations share ids,
            # but the composed children have an isolated id space.
            (LOAD_FAST, 'eval_'),
            (LOAD_CONST, op_code),
            (LOAD_FAST, 'toolkit'),
            (LOAD_FAST, 'f_globals'),
            (CALL_FUNCTION, 0x0003),
            (LOAD_ATTR, '__enaml_call__'),
            (LOAD_CONST, None),
            (LOAD_FAST, 'toolkit'),
            (CALL_FUNCTION, 0x0002),
            (STORE_FAST, name),
        ])
        
        if node.identifier:
            extend_ops([
                (LOAD_FAST, name),
                (LOAD_FAST, 'identifiers'),
                (LOAD_CONST, node.identifier),
                (STORE_SUBSCR, None),
            ])

        visit = self.visit
        for item in node.body:
            visit(item)
        
        self.pop_name()
        extend_ops([
            # foo.add_subcomponent(button)
            (LOAD_FAST, self.curr_name()),
            (LOAD_ATTR, 'add_subcomponent'),
            (LOAD_FAST, name),
            (CALL_FUNCTION, 0x0001),
            (POP_TOP, None),
        ])


#------------------------------------------------------------------------------
# Enaml Compiler
#------------------------------------------------------------------------------
class EnamlCompiler(_NodeVisitor):
    """ A compiler that will compile an enaml module ast node.
    
    The entry point is the `compile` classmethod which will compile
    the ast into an appropriate python object and place the results 
    in the provided module dictionary.

    """
    @classmethod
    def compile(cls, module_ast, filename):
        """ The main entry point of the compiler.

        Parameters
        ----------
        module_ast : Instance(enaml_ast.Module)
            The enaml module ast node that should be compiled.
        
        module_dict : dict
            The dictionary of the Python module into which we are
            compiling the enaml code.
        
        """
        compiler = cls(filename)
        compiler.visit(module_ast)

        module_ops = [(SetLineno, 1)]
        extend_ops = module_ops.extend

        # Generate the startup code for the module
        for start in STARTUP:
            start_code = compile(start, filename, mode='exec')
            # Skip the SetLineo and ReturnValue codes
            extend_ops(Code.from_code(start_code).code[1:-2])

        # Add in the code ops for the module
        extend_ops(compiler.code_ops)

        # Generate the cleanup code for the module
        for end in CLEANUP:
            end_code = compile(end, filename, mode='exec')
            # Skip the SetLineo and ReturnValue codes
            extend_ops(Code.from_code(end_code).code[1:-2])
        
        # Add in the final return value ops
        extend_ops([
            (LOAD_CONST, None),
            (RETURN_VALUE, None),
        ])

        # Generate and return the module code object.
        mod_code = Code(
            module_ops, [], [], False, False, False, '',  filename, 0, '',
        )
        return mod_code.to_code()

    def __init__(self, filename):
        self.code_ops = []
        self.filename = filename

    def visit_Module(self, node):
        """ The module node visitor method. Used internally by the
        compiler.

        """
        for item in node.body:
            self.visit(item)
    
    def visit_Python(self, node):
        """ A visitor which adds a chunk of raw Python into the module.

        """
        # This compiles the given Python ast into a Python code object
        # then disassembles it into a byteplay code object. This allows
        # us to interleave the instructions with those generated for
        # the rest of the module and then compile a single unified 
        # code object.
        py_code = compile(node.py_ast, self.filename, mode='exec')
        bpc = Code.from_code(py_code)
        # Skip the SetLineo and ReturnValue codes
        self.code_ops.extend(bpc.code[1:-2])

    def visit_Declaration(self, node):
        """ The declaration node visitor. This will add an instance
        of EnamlDeclaration to the module.

        """
        # This creates a function from the generated code ops then
        # wraps that function in an EnamlDeclaration.
        func_code = DeclarationCompiler.compile(node, self.filename)
        name = node.name
        self.code_ops.extend([
            (LOAD_CONST, func_code),
            (MAKE_FUNCTION, 0),
            (STORE_NAME, name),
            (LOAD_NAME, 'EnamlDeclaration'),
            (LOAD_NAME, name),
            (CALL_FUNCTION, 0x0001),
            (STORE_NAME, name),
        ])




#------------------------------------------------------------------------------
# Copyright (c) 2011, Enthought, Inc.
# All rights reserved.
#------------------------------------------------------------------------------
from .qt import QtGui
from .qt_base_widget_component import QtBaseWidgetComponent

from ...components.base_selection_model import AbstractTkBaseSelectionModel


_SELECTION_MODE_MAP = {
    'single': QtGui.QAbstractItemView.SingleSelection,
    'contiguous': QtGui.QAbstractItemView.ContiguousSelection,
    'extended': QtGui.QAbstractItemView.ExtendedSelection,
    'multi': QtGui.QAbstractItemView.MultiSelection,
    'none': QtGui.QAbstractItemView.NoSelection,
}


_SELECTION_BEHAVIOR_MAP = {
    'items' : QtGui.QAbstractItemView.SelectItems,
    'rows' : QtGui.QAbstractItemView.SelectRows,
    'columns' : QtGui.QAbstractItemView.SelectColumns,
}


_SELECTION_COMMAND_MAP = {
    'clear_select': QtGui.QItemSelectionModel.ClearAndSelect,
    'no_update': QtGui.QItemSelectionModel.NoUpdate,
    'clear': QtGui.QItemSelectionModel.Clear,
    'select': QtGui.QItemSelectionModel.Select,
    'deselect': QtGui.QItemSelectionModel.Deselect,
    'toggle': QtGui.QItemSelectionModel.Toggle,
    'current': QtGui.QItemSelectionModel.Current,
    'rows': QtGui.QItemSelectionModel.Rows,
    'columns': QtGui.QItemSelectionModel.Columns,
    'select_current': QtGui.QItemSelectionModel.SelectCurrent,
    'toggle_current': QtGui.QItemSelectionModel.ToggleCurrent,
}


class QtBaseSelectionModel(QtBaseWidgetComponent, AbstractTkBaseSelectionModel):
    """ Qt implementation of the BaseSelectionModel.

    """
    #--------------------------------------------------------------------------
    # Setup Methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Create a space for the underlying QItemSelectionModel.

        We don't want to actually get it yet, since it depends on the
        AbstractItemModel being set on the parent, which won't happen 
        until the initialize() step.

        """
        self.widget = None

    def initialize(self):
        """ Get the QItemSelectionModel.

        """
        super(QtBaseSelectionModel, self).initialize()
        shell = self.shell_obj
        self.set_selection_mode(shell.selection_mode)
        self.set_selection_behavior(shell.selection_behavior)

    def bind(self):
        """ Bind to events.

        """
        super(QtBaseSelectionModel, self).bind()
        parent = self.shell_obj.parent
        parent.on_trait_change(self.reset_for_new_model, 'item_model')
        selection_model = self.selection_model
        selection_model.currentChanged.connect(self._update_current)
        selection_model.selectionChanged.connect(self._update_selection)

    def reset_for_new_model(self):
        """ Reset the state for a new AbstractItemModel.

        """
        self.initialize()
        parent = self.shell_obj.parent
        handler = self.reset_for_new_model
        parent.on_trait_change(handler, 'item_model', remove=True)
        self.bind()

    @property
    def selection_model(self):
        """ Get the QItemSelectionModel for the parent widget.

        """
        shell = self.shell_obj
        qitem_selection_model = shell.parent.toolkit_widget.selectionModel()
        return qitem_selection_model

    @property
    def item_model(self):
        """ Get the AbstractItemModelWrapper for the corresponding item 
        model.

        """
        # FIXME: We must get it from the widget itself instead of the
        # QItemSelectionModel.model() member. There seems to be a bug 
        # in PySide causing segfaults on finalization otherwise.
        return self.shell_obj.parent.toolkit_widget.model()

    def py_selection_to_qt(self, selection):
        """ Converts a list of tuples of Enaml ModelIndex ranges into
        a QItemSelection.

        """
        qsel = QtGui.QItemSelection()
        qitem_model = self.item_model
        for topleft, botright in selection:
            qtopleft = qitem_model.to_q_index(topleft)
            qbotright = qitem_model.to_q_index(botright)
            qsel.select(qtopleft, qbotright)
        return qsel

    def qt_selection_to_py(self, qselection):
        """ Converts a QItemSelection into a list of tuples of Enaml
        ModelIndex ranges.

        """
        pysel = []
        qitem_model = self.item_model
        for qrange in qselection:
            topleft = qitem_model.from_q_index(qrange.topLeft())
            botright = qitem_model.from_q_index(qrange.bottomRight())
            pysel.append((topleft, botright))
        return pysel

    def _update_current(self, current, previous):
        qitem_model = self.item_model
        old = qitem_model.from_q_index(previous)
        new = qitem_model.from_q_index(current)
        self.shell_obj.current_event((old, new))

    def _update_selection(self, current, previous):
        old = self.qt_selection_to_py(previous)
        new = self.qt_selection_to_py(current)
        self.shell_obj.selection_event((old, new))

    def clear(self):
        """ Clear the selection and the current index.

        """
        self.selection_model.clear()

    def get_current_index(self):
        """ Return the current ModelIndex or None if there is not one.

        """
        qindex = self.selection_model.currentIndex()
        index = self.item_model.from_q_index(qindex)
        return index

    def set_current_index(self, index):
        """ Set the current ModelIndex.

        """
        qindex = self.item_model.to_q_index(index)
        self.selection_model.setCurrentIndex(qindex)

    def set_selection(self, selection, command='clear_select'):
        """ Set the current selection.

        """
        if isinstance(command, basestring):
            command = (command,)
        qflag = 0
        for cmd in command:
            qflag |= _SELECTION_COMMAND_MAP[cmd]
        qsel = self.py_selection_to_qt(selection)
        self.selection_model.select(qsel, qflag)

    def get_selection(self):
        """ Get the current selection.

        """
        qsel = self.selection_model.selection()
        pysel = self.qt_selection_to_py(qsel)
        return pysel

    def set_selection_mode(self, selection_mode):
        """ Sets the selection mode.

        """
        shell = self.shell_obj
        behavior = _SELECTION_MODE_MAP[selection_mode]
        item_widget = shell.parent.toolkit_widget
        item_widget.setSelectionMode(behavior)

    def set_selection_behavior(self, selection_behavior):
        """ Sets the selection behavior.

        """
        shell = self.shell_obj
        behavior = _SELECTION_BEHAVIOR_MAP[selection_behavior]
        item_widget = shell.parent.toolkit_widget
        item_widget.setSelectionBehavior(behavior)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from traits.api import TraitError

from .enaml_test_case import EnamlTestCase, required_method


class TestProgressBar(EnamlTestCase):
    """ Logic for testing the ProgressBar.

    Tooklit testcases need to provide the following functions

    Abstract Methods
    ----------------
    get_value(self, widget)
        Get the toolkits widget's active value.

    get_minimum(self, widget)
        Get the toolkits widget's minimum value.

    get_maximum(self, widget)
        Get the toolkits widget's maximum value.

    """

    def setUp(self):
        """ Set up before the ProgressBar tests.

        """

        enaml = """
enamldef MainView(MainWindow):
    ProgressBar:
        name = 'test'
"""

        self.view = self.parse_and_create(enaml)
        self.component = self.component_by_name(self.view, 'test')
        self.widget = self.component.toolkit_widget

    def test_initialization_values(self):
        """ Test the initial attributes of the ProgressBar component.

        """
        component = self.component

        self.assertEnamlInSync(component, 'value', 0)
        self.assertEnamlInSync(component, 'minimum', 0)
        self.assertEnamlInSync(component, 'maximum', 100)

    def test_change_minimum(self):
        """ Test changing the minimum value.

        """
        component = self.component
        new_minimum = 10
        component.minimum = new_minimum
        self.assertEnamlInSync(component, 'minimum', new_minimum)

    def test_change_maximum(self):
        """ Test changing the maximum value.

        """
        component = self.component
        new_maximum = 90
        component.maximum = new_maximum
        self.assertEnamlInSync(component, 'maximum', new_maximum)

    def test_change_value_in_enaml(self):
        """ Test changing the current value through the component.

        """
        component = self.component
        new_value = 10
        component.value = new_value
        self.assertEnamlInSync(component, 'value', new_value)

    def test_invalid_minimum(self):
        """ Test changing to an invalid value below the minimum.

        """
        component = self.component
        with self.assertRaises(TraitError):
            component.value = -10
        self.assertEnamlInSync(component, 'value', 0)

    def test_invalid_maximum(self):
        """ Test changing to an invalid value above the maximum.

        """
        component = self.component
        with self.assertRaises(TraitError):
            component.value = 110
        self.assertEnamlInSync(component, 'value', 0)

    def test_change_maximum_and_value(self):
        """ Test setting maximum while the value is out of range.

        """
        component = self.component
        component.value = 90
        component.maximum = 80
        self.assertEnamlInSync(component, 'value', 80)

    def test_change_minimum_and_value(self):
        """ Test setting minimum while the value is out of range.

        """
        component = self.component
        component.minimum = 10
        self.assertEnamlInSync(component, 'value', 10)

    def test_change_range_invalid(self):
        """ Test setting minimum > maximum.

        """
        component = self.component
        with self.assertRaises(TraitError):
            component.maximum = -10

        with self.assertRaises(TraitError):
            component.minimum = 110

    #--------------------------------------------------------------------------
    # Special initialization tests
    #--------------------------------------------------------------------------
    def test_initial_too_large(self):
        """ Check initialization with a value too large for the specified
        maximum.

        """
        enaml = """
enamldef MainView(MainWindow):
    ProgressBar:
        name = 'test'
        value = 95
        minimum = 10
        maximum = 90
"""
        # FIXME: need make a more refined check, this is not the best way
        with self.assertRaises(TraitError):
            self.parse_and_create(enaml)

    def test_initial_too_small(self):
        """ Check initialization with a value too small for the specified
        minimum.

        """

        enaml = """
enamldef MainView(MainWindow):
    ProgressBar:
        name = 'test'
        value = 5
        minimum = 10
        maximum = 90
"""

        # FIXME: need make a more refined check, this is not the best way
        with self.assertRaises(TraitError):
            self.parse_and_create(enaml)

    def test_percentage(self):
        """ Test that the percentage is computed accurately.

        """
        self.component.maximum = 1000
        self.component.value = 999
        # Note the rounding down to 99%.
        self.assertEquals(self.component.percentage, 99)
        self.component.value = 1000
        self.assertEquals(self.component.percentage, 100)
        self.component.value = 899
        # Note the rounding to the nearest integer.
        self.assertEquals(self.component.percentage, 90)
        self.component.value = 0
        self.component.maximum = 0
        self.assertEquals(self.component.percentage, 0)
        self.component.minimum = -1000
        self.assertEquals(self.component.percentage, 100)
        self.component.value = -1
        self.assertEquals(self.component.percentage, 99)

    #--------------------------------------------------------------------------
    # Abstract methods
    #--------------------------------------------------------------------------
    @required_method
    def get_value(self, widget):
        """  Get the toolkits widget's active value.

        """
        pass

    @required_method
    def get_minimum(self, widget):
        """  Get the toolkits widget's maximum value attribute.

        """
        pass

    @required_method
    def get_maximum(self, widget):
        """ Get the toolkits widget's minimum value attribute.

        """
        pass




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import sys
import unittest

from enaml.core import import_hooks


class FakeSys(object):
    def __init__(self):
        self.meta_path = []


class TestImportHooks(unittest.TestCase):

    def setUp(self):
        # Monkey-patch "sys" in the import_hooks module.
        import_hooks.sys = FakeSys()

    def tearDown(self):
        import_hooks.sys = sys

    def test_install_count(self):
        """ Test that EnamlImporter can be installed in a nested manner.

        """
        importer = import_hooks.EnamlImporter
        counts = importer._install_count
        meta_path = import_hooks.sys.meta_path

        self.assertEquals(counts[importer], 0)
        self.assertEquals(len(meta_path), 0)

        importer.install()
        self.assertEquals(counts[importer], 1)
        self.assertEquals(len(meta_path), 1)

        importer.install()
        self.assertEquals(counts[importer], 2)
        self.assertEquals(len(meta_path), 1)

        importer.uninstall()
        self.assertEquals(counts[importer], 1)
        self.assertEquals(len(meta_path), 1)

        importer.uninstall()
        self.assertEquals(counts[importer], 0)
        self.assertEquals(len(meta_path), 0)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import Instance, Property, Bool

from .base_widget_component import (
    BaseWidgetComponent, AbstractTkBaseWidgetComponent,
)

from ..core.trait_types import EnamlEvent
from ..guard import guard
from ..styling.color import ColorTrait
from ..styling.font import FontTrait


#------------------------------------------------------------------------------
# Freeze Context
#------------------------------------------------------------------------------
class FreezeContext(object):
    """ A context manager which disables rendering updates of a component
    on enter, and re-enables them on exit. It may safely nested.

    """
    _counts = {}

    @classmethod
    def is_frozen(cls, component):
        """ Returns a boolean indicating whether or not the given 
        component is currently the subject of a freeze context.

        """
        return component in cls._counts

    def __init__(self, component):
        """ Initialize a freeze context.

        Parameters
        ----------
        component : BaseComponent
            The component that should be frozen.
        
        """
        self.component = component
    
    def __enter__(self):
        """ Disables updates on the component the first time that any
        freeze context on that component is entered.

        """
        counts = self._counts
        cmpnt = self.component
        if cmpnt not in counts:
            counts[cmpnt] = 1
            cmpnt.disable_updates()
        else:
            counts[cmpnt] += 1
    
    def __exit__(self, exc_type, exc_value, traceback):
        """ Enables updates on the component when the last freeze
        context on that component is exited.

        """
        counts = self._counts
        cmpnt = self.component
        counts[cmpnt] -= 1
        if counts[cmpnt] == 0:
            del counts[cmpnt]
            cmpnt.enable_updates()


#------------------------------------------------------------------------------
# Abstract Toolkit Widget Component Interface
#------------------------------------------------------------------------------
class AbstractTkWidgetComponent(AbstractTkBaseWidgetComponent):
    """ The abstract toolkit WidgetComponent interface.

    This abstract interface adds support for widgets that can paint on
    the screen, and thus have associated geometry and style information.

    """
    @abstractmethod
    def disable_updates(self):
        """ Called when the widget should disable its rendering updates.

        This is used before a large change occurs to a live ui so that
        the widget can optimize/delay redrawing until all the updates
        have been completed and 'enable_updates' is called.

        """
        raise NotImplementedError

    @abstractmethod
    def enable_updates(self):
        """ Called when the widget should enable rendering updates.

        This is used in conjunction with 'disable_updates' to allow
        a widget to optimize/delay redrawing until all the updates
        have been completed and this function is called.

        """
        raise NotImplementedError

    @abstractmethod
    def set_visible(self, visible):
        """ Set the visibility of the of the widget according to the
        given boolean.

        """
        raise NotImplementedError

    @abstractmethod
    def size_hint(self):
        """ Returns the size hint Size tuple as given by the abstract 
        widget for its current state.

        """
        raise NotImplementedError

    @abstractmethod
    def layout_geometry(self):
        """ Returns the Rect tuple of layout geometry info as given by 
        the abstract object. It may be different from the value returned 
        by geometry() if the widget's effective layout rect is different 
        from its paintable rect.

        """
        raise NotImplementedError

    @abstractmethod
    def set_layout_geometry(self, rect):
        """ Sets the layout geometry of the internal widget to the 
        given Rect.

        """
        raise NotImplementedError

    @abstractmethod
    def geometry(self):
        """ Returns the Rect geometry tuple as given by the abstract 
        widget.

        """
        raise NotImplementedError
    
    @abstractmethod
    def set_geometry(self, rect):
        """ Sets the geometry of the abstract widget with the given
        Rect.

        """
        raise NotImplementedError

    @abstractmethod
    def min_size(self):
        """ Returns the hard minimum Size of the widget, ignoring any 
        windowing decorations. A widget will not be able to be resized
        smaller than this value

        """
        raise NotImplementedError

    @abstractmethod
    def set_min_size(self, size):
        """ Set the hard minimum Size of the widget, ignoring any 
        windowing decorations. A window will not be able to be resized 
        smaller than this value.

        """
        raise NotImplementedError

    @abstractmethod
    def max_size(self):
        """ Returns the hard maximum Size of the widget, ignoring any
        windowing decorations. A widget will not be able to be resized
        larger than this value

        """
        raise NotImplementedError

    @abstractmethod
    def set_max_size(self, size):
        """ Set the hard maximum Size of the widget, ignoring any 
        windowing decorations. A widget will not be able to be resized 
        larger than this value.

        """
        raise NotImplementedError

    @abstractmethod
    def size(self):
        """ Returns the Size as given by the abstract widget.

        """
        raise NotImplementedError
        
    @abstractmethod
    def resize(self, size):
        """ Resize the abstract widget as specified by the given Size.

        """
        raise NotImplementedError

    @abstractmethod
    def pos(self):
        """ Returns the Pos tuple as given by the abstract widget.

        """
        raise NotImplementedError
    
    @abstractmethod
    def move(self, pos):
        """ Moves the abstract widget to the given Pos position relative
        to the its parent's origin

        """
        raise NotImplementedError

    @abstractmethod
    def shell_enabled_changed(self, enabled):
        """ The change handler for the 'enabled' attribute of the shell
        object. Sets the widget enabled according to the given boolean.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_bgcolor_changed(self, color):
        """ The change handler for the 'bgcolor' attribute on the shell
        object. Sets the background color of the internal widget to the 
        given color.
        
        """
        raise NotImplementedError
    
    @abstractmethod
    def shell_fgcolor_changed(self, color):
        """ The change handler for the 'fgcolor' attribute on the shell
        object. Sets the foreground color of the internal widget to the 
        given color. For some widgets this may do nothing.

        """
        raise NotImplementedError
    
    @abstractmethod
    def shell_font_changed(self, font):
        """ The change handler for the 'font' attribute on the shell
        object. Sets the font of the internal widget to the given font.
        For some widgets this may do nothing.

        """
        raise NotImplementedError


#------------------------------------------------------------------------------
# Widget Component
#------------------------------------------------------------------------------
class WidgetComponent(BaseWidgetComponent):
    """ A BaseWidgetComponent subclass that adds support for a visual
    toolkit widget information.

    """
    #: Whether or not the widget is enabled.
    enabled = Bool(True)

    #: Whether or not the widget is visible.
    visible = Bool(True)

    #: The background color of the widget.
    bgcolor = ColorTrait

    #: The foreground color of the widget.
    fgcolor = ColorTrait

    #: The font used for the widget.
    font = FontTrait

    #: A property which returns whether or not this component is 
    #: currently frozen.
    frozen = Property

    #: An event that should be emitted when the size hint been updated.
    size_hint_updated = EnamlEvent

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkWidgetComponent) 

    #--------------------------------------------------------------------------
    # Property Getters
    #--------------------------------------------------------------------------
    def _get_frozen(self):
        """ The property getter for the 'frozen' attribute.

        """
        return FreezeContext.is_frozen(self)

    #--------------------------------------------------------------------------
    # Setup Methods
    #--------------------------------------------------------------------------
    def _setup_init_visibility(self):
        """ A reimplemented parent class setup method which initializes
        the visibility of the component.

        """
        self.set_visible(self.visible)
        super(WidgetComponent, self)._setup_init_visibility()
    
    #--------------------------------------------------------------------------
    # Change Handlers
    #--------------------------------------------------------------------------
    def _visible_changed(self, visible):
        """ Handles the 'visible' attribute being changed by calling the
        'set_visible' method and requesting the parent.

        """
        # The method call allows the visibility to be set by other parts 
        # of the framework which are not dependent on change notification.
        # By using the method here, we help to ensure a consistent state.
        self.set_visible(visible)

    def _size_hint_updated_changed(self):
        """ A change handler which requests a relayout from its *parent*
        when its size hint has updated, provided that the component is
        initialized.

        Note that it is critical that the request be made to the parent,
        since a given component has no use for its size hint in its 
        internal layout computations.

        """
        if self.initialized:
            parent = self.parent
            if parent is not None:
                parent.request_relayout()

    #--------------------------------------------------------------------------
    # Geometry Methods
    #--------------------------------------------------------------------------
    def size_hint(self):
        """ Returns the size hint Size tuple as given by the abstract 
        widget for its current state.

        """
        return self.abstract_obj.size_hint()

    def layout_geometry(self):
        """ Returns the Rect tuple of layout geometry info as given by 
        the abstract object. It may be different from the value returned 
        by geometry() if the widget's effective layout rect is different 
        from its paintable rect.

        """
        return self.abstract_obj.layout_geometry()

    def set_layout_geometry(self, rect):
        """ Sets the layout geometry of the internal widget to the 
        given Rect.

        """
        self.abstract_obj.set_layout_geometry(rect)

    def geometry(self):
        """ Returns the Rect geometry tuple as given by the abstract 
        widget.

        """
        return self.abstract_obj.geometry()
    
    def set_geometry(self, rect):
        """ Sets the geometry of the abstract widget with the given
        Rect.

        """
        self.abstract_obj.set_geometry(rect)

    def min_size(self):
        """ Returns the hard minimum Size of the widget, ignoring any 
        windowing decorations. A widget will not be able to be resized
        smaller than this value

        """
        return self.abstract_obj.min_size()

    def set_min_size(self, size):
        """ Set the hard minimum Size of the widget, ignoring any 
        windowing decorations. A window will not be able to be resized 
        smaller than this value.

        """
        self.abstract_obj.set_min_size(size)

    def max_size(self):
        """ Returns the hard maximum Size of the widget, ignoring any
        windowing decorations. A widget will not be able to be resized
        larger than this value

        """
        return self.abstract_obj.max_size()

    def set_max_size(self, size):
        """ Set the hard maximum Size of the widget, ignoring any 
        windowing decorations. A widget will not be able to be resized 
        larger than this value.

        """
        self.abstract_obj.set_max_size(size)

    def size(self):
        """ Returns the Size as given by the abstract widget.

        """
        return self.abstract_obj.size()

    def resize(self, size):
        """ Resize the abstract widget as specified by the given Size.

        """
        self.abstract_obj.resize(size)

    def pos(self):
        """ Returns the Pos tuple as given by the abstract widget.

        """
        return self.abstract_obj.pos()
    
    def move(self, pos):
        """ Moves the abstract widget to the given Pos position relative
        to the its parent's origin

        """
        self.abstract_obj.move(pos)
    
    #--------------------------------------------------------------------------
    # Auxiliary Methods 
    #--------------------------------------------------------------------------
    def set_visible(self, visible):
        """ Set the visibility of the component according to the given
        boolean. Subclasses that need more control over visibility
        changes should reimplement this method.

        """
        # Make sure the 'visible' attribute is synced up as a result
        # of the method call. This may fire a notification, in which
        # case the change handler will call this method again. This
        # guard prevents that unneeded recursion.
        if guard.guarded(self, 'set_visible'):
            return
        else:
            with guard(self, 'set_visible'):
                self.visible = visible
        
        # Only set the visibility of the component if it is fully 
        # initialized or being set to False. This prevents situations 
        # where a widget is shown prematurely in some toolkit backends
        # which then causes the entire window hierarchy to be shown
        # prematurely.
        #
        # XXX this is still a bit gorpy and needs some work.
        if self.initialized:
            self.request_relayout_task(self.abstract_obj.set_visible, visible)
        elif not visible or self.parent is not None:
            self.abstract_obj.set_visible(visible)

    def disable_updates(self):
        """ Disables rendering updates for the underlying widget.

        """
        self.abstract_obj.disable_updates()

    def enable_updates(self):
        """ Enables rendering updates for the underlying widget.

        """
        self.abstract_obj.enable_updates()

    def freeze(self):
        """ A context manager which disables rendering updates on 
        enter and restores them on exit. The context can be safetly
        nested.

        """
        return FreezeContext(self)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod
from datetime import date

from traits.api import Date, Instance, Property, TraitError, on_trait_change

from .control import Control, AbstractTkControl

from ..core.trait_types import Bounded


class AbstractTkBoundedDate(AbstractTkControl):
    """ The abstract toolkit BoundedDate interface.

    """
    @abstractmethod
    def shell_date_changed(self, date):
        """ The change handler for the 'date' attribute on the shell
        object.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_min_date_changed(self, min_date):
        """ The change handler for the 'min_date' attribute on the shell
        object.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_max_date_changed(self, max_date):
        """ The change handler for the 'max_date' attribute on the shell
        object.

        """
        raise NotImplementedError


class BoundedDate(Control):
    """ A base class for use with widgets that edit a Python 
    datetime.date object bounded between minimum and maximum 
    values. This class is not meant to be used directly.

    """
    #: The minimum date available in the date edit. If not defined then
    #: the default value is September 14, 1752. Extra checks take place 
    #: to make sure that the user does not programmatically set
    #: :attr:`min_date` > :attr:`max_date`.
    min_date = Property(Date, depends_on ='_min_date')

    #: The internal min date storage
    _min_date = Date(date(1752, 9, 14))

    #: The maximum date available in the date edit. If not defined then
    #: the default value is December 31, 7999. Extra checks take place 
    #: to make sure that the user does not programmatically set
    #: :attr:`min_date` > :attr:`max_date`.
    max_date = Property(Date, depends_on ='_max_date')

    #: The internal max date storage
    _max_date = Date(date(7999, 12, 31))

    #: The currently selected date. Default is the current date. The
    #: value is bounded between :attr:`min_date` and :attr:`max_date`. 
    #: Changing the boundary attributes might result in an update of 
    #: :attr:`date` to fit in the new range. Attempts to assign a value 
    #: outside of these bounds will result in a TraitError.
    date = Bounded(Date(date.today()), low='min_date', high='max_date')

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkBoundedDate)

    #--------------------------------------------------------------------------
    # Property methods
    #--------------------------------------------------------------------------
    def _set_min_date(self, date):
        """ Set the min_date. Addtional checks are applied to make sure 
        that :attr:`min_date` < :attr:`max_date`

        """
        if date > self.max_date:
            msg = ("The minimum date should be smaller than the current "
                   "maximum date({0}), but a value of {1} was given.")
            msg = msg.format(self.max_date, date)
            raise TraitError(msg)
        self._min_date = date

    def _set_max_date(self, date):
        """ Set the max_date. Addtional checks are applied to make sure
        that :attr:`min_date` < :attr:`max_date`

        """
        if date < self.min_date:
            msg = ("The maximum date should be larger than the current "
                   "minimum date({0}), but a value of {1} was given.")
            msg = msg.format(self.min_date, date)
            raise TraitError(msg)
        self._max_date = date

    def _get_max_date(self):
        """ The property getter for the maximum date.

        """
        return self._max_date

    def _get_min_date(self):
        """ The property getter for the minimum date.

        """
        return self._min_date
    
    @on_trait_change('min_date, max_date')
    def _adapt_date(self):
        """ Adapt the date to the bounderies

        """
        if self.initialized:
            self.date = min(max(self.date, self.min_date), self.max_date)




#------------------------------------------------------------------------------
# Copyright (c) 2011, Enthought, Inc.
# All rights reserved.
#------------------------------------------------------------------------------
from traits.api import List, Int, on_trait_change

from .base_selection_model import BaseSelectionModel

from ..guard import guard


class RowSelectionModel(BaseSelectionModel):
    """ A selection model that maps to a list of row indices.

    Generally, this should be used with `selection_behavior='rows'`.

    """
    #: The selected row indices.
    selected_rows = List(Int)

    #: Only select rows.
    selection_behavior = 'rows'

    @on_trait_change('selection_event')
    def _update_rows(self, event):
        selected_rows = []
        for topleft, botright in self.get_selection():
            selected_rows.extend(range(topleft.row, botright.row+1))
        selected_rows.sort()
        with guard(self, self._update_rows):
            self.selected_rows = selected_rows

    @on_trait_change('selected_rows,selected_rows_items')
    def _update_selection(self):
        if guard.guarded(self, self._update_rows):
            return
        item_model = self.parent.item_model
        selection = []
        # FIXME: try to merge contiguous rows into ranges.
        for i in self.selected_rows:
            topleft = item_model.create_index(i, 0, item_model)
            botright = item_model.create_index(i, 0, item_model)
            selection.append((topleft, botright))
        self.set_selection(selection, ('clear_select', 'rows'))




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt_slider import QtSlider

from ...components.float_slider import AbstractTkFloatSlider


class QtFloatSlider(QtSlider, AbstractTkFloatSlider):
    """ A Qt implementation of FloatSlider.

    """
    def initialize(self):
        """ Initializes the attributes of the toolkit widget.

        """
        shell = self.shell_obj
        shell._down = False
        self.set_precision(shell.precision)
        super(QtFloatSlider, self).initialize()

    def bind(self):
        """ Connect the event handlers for the slider widget signals.

        """
        super(QtFloatSlider, self).bind()
        widget = self.widget
        widget.valueChanged.connect(self._on_slider_changed)
        widget.sliderMoved.connect(self._on_slider_moved)
        widget.sliderPressed.connect(self._on_pressed)
        widget.sliderReleased.connect(self._on_released)

    #--------------------------------------------------------------------------
    # Implementation
    #--------------------------------------------------------------------------

    def shell_precision_changed(self, precision):
        """ The change handler for the 'precision' attribute on the shell
        object.
        
        """
        self.set_precision(precision)

    #--------------------------------------------------------------------------
    # Event Handlers 
    #--------------------------------------------------------------------------
    def _on_slider_changed(self):
        """ The event handler for the slider value changed event.

        """
        self.shell_obj.value = self.int_to_float(self.widget.value())

    def _on_slider_moved(self):
        """ The event handler for a slider moved event.

        """
        self.shell_obj.moved(self.int_to_float(self.widget.sliderPosition()))

    #--------------------------------------------------------------------------
    # Widget Update Methods 
    #--------------------------------------------------------------------------
    def set_precision(self, precision):
        """ Set the precision attribute in the Qt widget.

        """
        self.widget.setRange(0, precision)

    def set_range(self, minimum, maximum):
        """ Set the range of the slider widget.

        Overridden to ignore the float values. The Qt widget knows nothing about
        them.

        """
        pass

    def set_position(self, value):
        """ Validate the position value.

        """
        self.widget.setValue(self.float_to_int(value))

    def float_to_int(self, value):
        """ Convert a float value to an int value.

        """
        shell = self.shell_obj
        u = (value - shell.minimum) / (shell.maximum - shell.minimum)
        i = int(round(u * shell.precision))
        return i

    def int_to_float(self, value):
        """ Convert an int value to a float value.

        """
        shell = self.shell_obj
        u = float(value) / shell.precision
        x = u * shell.maximum + (1.0-u) * shell.minimum
        return x



#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from traits.api import HasTraits, Any, on_trait_change

from chaco.api import (ArrayPlotData, ArrayDataSource, DataRange1D, 
                       LinearMapper, VPlotContainer, PlotAxis, FilledLinePlot, 
                       add_default_grids, Plot)
from chaco.tools.api import RangeSelection
from chaco.scales.api import CalendarScaleSystem
from chaco.scales_tick_generator import ScalesTickGenerator

from zoom_overlay import ZoomOverlay


class PlotDriver(HasTraits):

    model = Any

    def __init__(self, model):
        super(PlotDriver, self).__init__(model=model)

    @on_trait_change('model:data')
    def update_plots(self, data):
        self.close_plot.index.set_data(data['dates'])
        self.close_plot.value.set_data(data['close'])
        self.low_plot.index.set_data(data['dates'])
        self.low_plot.value.set_data(data['low'])
        self.bar_plot.index.set_data(data['dates'])
        self.bar_plot.value.set_data(data['volume'])
        self.close_plot.controller.deselect()

    def get_plot_component(self):
        # Create the array plot data that will feed our plots
        data = self.model.data
        plot_data = ArrayPlotData(index=data['dates'], 
                                  close=data['close'],
                                  volume=data['volume'],)
        self.plot_data = plot_data
        
        # Need to make the FilledLinePlot manually since Plot doesn't
        # support that plot type.
        times = ArrayDataSource(data['dates'])
        prices = ArrayDataSource(data['close'])
        close_plot =  FilledLinePlot(index=times, value=prices,
                        index_mapper = LinearMapper(range=DataRange1D(times)),
                        value_mapper = LinearMapper(range=DataRange1D(prices)),
                        edge_color = 'blue',
                        face_color = 'paleturquoise',
                        bgcolor = 'white',
                        border_visible = True)
        close_plot.padding = [40, 15, 15, 20]
        self.close_plot = close_plot

        # The second plotter object which generates our candle plot
        plotter2 = Plot(data=plot_data)
        low_plot = plotter2.plot(('index', 'close'),)[0]
        low_plot.height = 100
        low_plot.resizable = 'h'
        low_plot.bgcolor = 'white'
        low_plot.border_visible = True
        low_plot.padding = [40, 15, 15, 20]
        low_plot.color = 'darkred'
        low_plot.line_width = 1.5
        self.low_plot = low_plot

        # The third plotter for the bar plot.
        plotter3 = Plot(data=plot_data)
        bar_plot = plotter3.plot(('index', 'volume'), type='bar')[0]
        bar_plot.height = 100
        bar_plot.resizable = 'h'
        bar_plot.bgcolor = 'white'
        bar_plot.border_visible = True
        bar_plot.padding = [40, 15, 15, 20]
        bar_plot.line_color = 'transparent'
        bar_plot.fill_color = 'black'
        bar_plot.bar_width = 3.0
        bar_plot.bar_width_type = 'screen'
        bar_plot.antialias = False
        bar_plot.index_mapper = low_plot.index_mapper
        self.bar_plot = bar_plot
    
        for plot in (close_plot, low_plot, bar_plot):
            ticker = ScalesTickGenerator(scale=CalendarScaleSystem())
            bottom_axis = PlotAxis(plot, orientation='bottom', 
                                   tick_generator=ticker)
            plot.overlays.append(bottom_axis)
            plot.overlays.append(PlotAxis(plot, orientation='left'))
            hgrid, vgrid = add_default_grids(plot)
            vgrid.tick_generator = bottom_axis.tick_generator
        
        def vol_label_formatter(val):
            return '%.1E' % val
        
        bar_plot.overlays[-1].tick_label_formatter = vol_label_formatter

        container = VPlotContainer(
            bgcolor=(240/255., 240/255., 240/255., 1.0),
            spacing=20,
            padding=20,
            fill_padding=True,
            stack_order='top_to_bottom',
            use_back_buffer=True,
        )

        container.add(close_plot)
        container.add(low_plot)
        container.add(bar_plot)

        close_plot.controller = RangeSelection(close_plot)
        zoom_overlay = ZoomOverlay(source=close_plot, destination=low_plot,
                                   other=bar_plot)
        container.overlays.append(zoom_overlay)

        return container




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt import QtGui, QtCore
from .qt_bounded_date import QtBoundedDate

from ...components.calendar import AbstractTkCalendar
from ...guard import guard


# Workaround for an incompatibility between PySide and PyQt
try: # pragma: no cover
    qdate_to_python = QtCore.QDate.toPython
except AttributeError: # pragma: no cover
    qdate_to_python = QtCore.QDate.toPyDate


class QtCalendar(QtBoundedDate, AbstractTkCalendar):
    """ A Qt implementation of Calendar.

    """
    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying QtCalendarWidget.

        """
        self.widget = QtGui.QCalendarWidget(parent)

    def bind(self):
        """ Binds the event handlers for the calendar widget.

        """
        super(QtCalendar, self).bind()
        widget = self.widget
        widget.activated.connect(self.on_date_activated)
        widget.selectionChanged.connect(self.on_date_selected)

    #--------------------------------------------------------------------------
    # Event Handlers
    #--------------------------------------------------------------------------
    def on_date_activated(self, qdate):
        """ The event handler for the calendar's activation event.

        """
        shell = self.shell_obj
        date = qdate_to_python(qdate)
        shell.date = date
        shell.activated(date)

    def on_date_selected(self):
        """ The event handler for the calendar's selection event.

        """
        if not guard.guarded(self, 'updating'):
            date = qdate_to_python(self.widget.selectedDate())
            self.shell_obj.selected(date)

    #--------------------------------------------------------------------------
    # Widget Update Methods
    #--------------------------------------------------------------------------
    def set_date(self, date):
        """ Sets and validates the component date on the widget.

        """
        # Calling setSelectedDate will trigger the dateChanged signal.
        # We want to avoid that feeback loop since the value is being 
        # set programatically.
        with guard(self, 'updating'):
            self.widget.setSelectedDate(date)

    def set_min_date(self, min_date):
        """ Sets the minimum date on the widget with the provided value.

        """
        # Calling setMinimumDate *may* trigger the dateChanged signal,
        # if the date needs to be clipped. We want to avoid that feeback 
        # loop since the value is being set programatically and the new
        # date will already have been updated by the shell object.
        with guard(self, 'updating'):
            self.widget.setMinimumDate(min_date)

    def set_max_date(self, max_date):
        """ Sets the maximum date on the widget with the provided value.

        """
        # Calling setMaximumDate *may* trigger the dateChanged signal,
        # if the date needs to be clipped. We want to avoid that feeback 
        # loop since the value is being set programatically and the new
        # date will already have been updated by the shell object.
        with guard(self, 'updating'):
            self.widget.setMaximumDate(max_date)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import Bool, Int, Str, Property, Instance

from .control import Control, AbstractTkControl

from ..core.trait_types import EnamlEvent


class AbstractTkTextEditor(AbstractTkControl):
    """ A text editor widget capable of editing plain text with styles.
    
    This is not a general text edit widget with general capabilties for 
    sophistcated formatting or image display.
    
    """
    @abstractmethod
    def shell_cursor_position_changed(self, cursor_position):
        raise NotImplementedError
    
    @abstractmethod
    def shell_anchor_position_changed(self, anchor_position):
        raise NotImplementedError
    
    @abstractmethod
    def shell_wrap_lines_changed(self, value):
        raise NotImplementedError
    
    @abstractmethod
    def shell_overwrite_changed(self, overwrite):
        raise NotImplementedError

    @abstractmethod
    def get_text(self):
        raise NotImplementedError

    @abstractmethod
    def set_text(self, text):
        raise NotImplementedError

    @abstractmethod
    def set_selection(self, start, end):
        raise NotImplementedError

    @abstractmethod
    def select_all(self):
        raise NotImplementedError

    @abstractmethod
    def deselect(self):
        raise NotImplementedError

    @abstractmethod
    def clear(self):
        raise NotImplementedError

    @abstractmethod
    def backspace(self):
        raise NotImplementedError

    @abstractmethod
    def delete(self):
        raise NotImplementedError

    @abstractmethod
    def end(self, mark=False):
        raise NotImplementedError

    @abstractmethod
    def home(self, mark=False):
        raise NotImplementedError

    @abstractmethod
    def cut(self):
        raise NotImplementedError

    @abstractmethod
    def copy(self):
        raise NotImplementedError

    @abstractmethod
    def paste(self):
        raise NotImplementedError

    @abstractmethod
    def insert(self, text):
        raise NotImplementedError

    @abstractmethod
    def undo(self):
        raise NotImplementedError

    @abstractmethod
    def redo(self):
        raise NotImplementedError
    
    @abstractmethod
    def find(self, text, backwards=False, case_sensitive=False, whole_words=False):
        """ Finds the text in the editor and sets the cursor to that 
        position.

        Parameters
        ----------
        text : string
            The text to find in the buffer

        backwards : bool, optional
            If True search starting at the end of the buffer. Defaults
            to False.
        
        case_sensitive : bool, optional
            If True, the text matching is case sensitive. Defaults to 
            False.
        
        whole_words : bool, optional
            If True, only complete words are matched. Defaults to False.

        Returns
        -------
        result : bool
            True if the text was found, False otherwise.
        
        """
        raise NotImplementedError


class TextEditor(Control):
    """ A text editor widget capable of editing plain text with styles.
    
    This is not a general text edit widget with general capabilties for 
    sophistcated formatting or image display.
    
    """
    #: Whether or not the editor is read only.
    read_only = Bool(False)
    
    #: Whether to wrap text in the editor or not. If True, keep words
    #: together if possible.
    wrap_lines = Bool(True)
    
    #: Whether to overwrite or insert text when the user types.
    overwrite = Bool(False)
    
    #: The position of the cursor in the editor.
    cursor_position = Int

    #: The anchor of the selection in the editor.
    anchor_position = Int

    #: A read only property holding the line number of cursor in the
    #: editor.
    cursor_line = Property(Int, depends_on='_cursor_line')

    #: A read only property holding the column number of cursor in the
    #: editor.
    cursor_column = Property(Int, depends_on='_cursor_column')

    #: A read only property that is set to True if the user has changed
    #: the line edit from the ui, False otherwise. This is reset to
    #: False if the text is programmatically changed.
    modified = Property(Bool, depends_on='_modified')

    #: A read only property that is updated with the text selected
    #: in the editor.
    selected_text = Property(Str, depends_on='_selected_text')

    #: Fired when the text is changed programmatically, or by the user
    #: via the ui. The event does not carry a payload. To retrieve the
    #: current text, use the `get_text()` method.
    text_changed = EnamlEvent

    #: Fired when the text is changed by the user explicitly through
    #: the ui and not programmatically. The event does not carry a 
    #: payload. To retrieve the current text, call `get_text()`.
    text_edited = EnamlEvent

    #: Fired when the widget has lost input focus.
    lost_focus = EnamlEvent

    #: How strongly a component hugs it's contents' width. TextEditors 
    #: ignore the width hug by default, so they expand freely in width.
    hug_width = 'ignore'
    
    #: How strongly a component hugs it's contents' height. TextEditors 
    #: ignore the height hug by default, so they expand freely in height.
    hug_height = 'ignore'

    #: An internal attribute that is used by the implementation object
    #: to update the value of 'cursor_line'.
    _cursor_line = Int

    #: An internal attribute that is used by the implementation object
    #: to update the value of 'cursor_column'.
    _cursor_column = Int

    #: An internal attribute that is used by the implementation object
    #: to update the value of 'modified'.
    _modified = Bool(False)

    #: An internal attribute that is used by the implementation object
    #: to update the value of 'selected_text'.
    _selected_text = Str

    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkTextEditor)

    def get_text(self):
        """ Get the current text as a string.

        """
        return self.abstract_obj.get_text()

    def set_text(self, text):
        """ Set the current text without an event notification.

        """
        return self.abstract_obj.set_text(text)

    def set_selection(self, start, end):
        """ Sets the selection to the bounds of start and end.

        If the indices are invalid, no selection will be made, and any
        current selection will be cleared.

        Arguments
        ---------
        start : Int
            The start selection index, zero based.

        end : Int
            The end selection index, zero based.

        """
        self.abstract_obj.set_selection(start, end)

    def select_all(self):
        """ Select all the text in the line edit.

        If there is no text in the line edit, the selection will be
        empty.

        """
        self.abstract_obj.select_all()

    def deselect(self):
        """ Deselect any selected text.

        """
        self.abstract_obj.deselect()

    def clear(self):
        """ Clear the line edit of all text.

        """
        self.abstract_obj.clear()

    def backspace(self):
        """ Simple backspace functionality.

        If no text is selected, deletes the character to the left of 
        the cursor. Otherwise, it deletes the selected text.

        """
        self.abstract_obj.backspace()

    def delete(self):
        """ Simple delete functionality.

        If no text is selected, deletes the character to the right of
        the cursor. Otherwise, it deletes the selected text.

        """
        self.abstract_obj.delete()

    def end(self, mark=False):
        """ Moves the cursor to the end of the line.

        Arguments
        ---------
        mark : bool, optional
            If True, select the text from the current position to the
            end of the line edit. Defaults to False.

        """
        self.abstract_obj.end(mark=mark)

    def home(self, mark=False):
        """ Moves the cursor to the beginning of the line.

        Arguments
        ---------
        mark : bool, optional
            If True, select the text from the current position to the 
            beginning of the line edit. Defaults to False.

        """
        self.abstract_obj.home(mark=mark)

    def cut(self):
        """ Cuts the selected text from the line edit.

        Copies the selected text to the clipthen deletes the selected
        text from the line edit.

        """
        self.abstract_obj.cut()

    def copy(self):
        """ Copies the selected text to the clipboard.

        """
        self.abstract_obj.copy()

    def paste(self):
        """ Paste the contents of the clipboard into the line edit.

        Inserts the contents of the clipboard into the line edit at
        the current cursor position, replacing any selected text.

        """
        self.abstract_obj.paste()

    def insert(self, text):
        """ Insert the text into the line edit.

        Inserts the given text at the current cursor position, replacing
        any selected text.

        Arguments
        ---------
        text : str
            The text to insert into the line edit.

        """
        self.abstract_obj.insert(text)

    def undo(self):
        """ Undoes the last operation.

        """
        self.abstract_obj.undo()

    def redo(self):
        """ Redoes the last operation.

        """
        self.abstract_obj.redo()

    def find(self, text, backwards=False, case_sensitive=False, whole_words=False):
        """ Finds the text in the editor and sets the cursor to that 
        position.

        Parameters
        ----------
        text : string
            The text to find in the buffer

        backwards : bool, optional
            If True search starting at the end of the buffer. Defaults
            to False.
        
        case_sensitive : bool, optional
            If True, the text matching is case sensitive. Defaults to 
            False.
        
        whole_words : bool, optional
            If True, only complete words are matched. Defaults to False.

        Returns
        -------
        result : bool
            True if the text was found, False otherwise.
        
        """
        return self.abstract_obj.find(text, backwards, case_sensitive, whole_words)

    #--------------------------------------------------------------------------
    # Property methods 
    #--------------------------------------------------------------------------
    def _get_cursor_column(self):
        """ The property getter for the 'cursor_column' attribute.

        """
        return self._cursor_column

    def _get_cursor_line(self):
        """ The property getter for the 'cursor_line' attribute.

        """
        return self._cursor_line

    def _get_modified(self):
        """ The property getter for the 'modified' attribute.

        """
        return self._modified

    def _get_selected_text(self):
        """ The property getter for the 'selected' attribute.

        """
        return self._selected_text
    



#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt.QtCore import QRect
from .qt.QtGui import QFrame, QWidgetItem, QApplication
from .qt_base_widget_component import QtBaseWidgetComponent
from .styling import q_color_from_color, q_font_from_font

from ...components.widget_component import AbstractTkWidgetComponent
from ...layout.geometry import Rect, Size, Pos


class QtWidgetComponent(QtBaseWidgetComponent, AbstractTkWidgetComponent):
    """ A Qt4 implementation of WidgetComponent.

    """
    #: A class attribte which indicates whether or not to use a 
    #: QWidget item to compute the layout geometry. Subclasses
    #: should override as necessary to change the behavior. The 
    #: default is True. 
    use_widget_item_for_layout = True

    @property
    def _widget_item(self):
        """ A readonly cached property which returns the QWidgetItem
        for the underlying Qt widget.

        """
        try:
            res = self.__widget_item
        except AttributeError:
            res = self.__widget_item = QWidgetItem(self.widget)
        return res

    def create(self, parent):
        """ Creates the underlying Qt widget. As necessary, subclasses
        should reimplement this method to create different types of
        widgets.

        """
        self.widget = QFrame(parent)

    def initialize(self):
        """ Initializes the attributes of the the Qt widget.

        """
        super(QtWidgetComponent, self).initialize()
        shell = self.shell_obj
        self.set_enabled(shell.enabled)
        if shell.bgcolor:
            self.set_bgcolor(shell.bgcolor)
        if shell.fgcolor:
            self.set_fgcolor(shell.fgcolor)
        if shell.font:
            self.set_font(shell.font)

    def enable_updates(self):
        """ Enable rendering updates for the underlying Wx widget.

        """
        # Freezing updates on a top-level window seems to cause 
        # flicker on OSX when the updates are reenabled. In this 
        # case, just freeze the children instead.
        if self.widget.isWindow():
            for child in self.shell_obj.children:
                child.enable_updates()
        self.widget.setUpdatesEnabled(True)
    
    def disable_updates(self):
        """ Disable rendering updates for the underlying Qt widget.

        """
        # Freezing updates on a top-level window seems to cause 
        # flicker on OSX when the updates are reenabled. In this 
        # case, just freeze the children instead.
        if self.widget.isWindow():
            for child in self.shell_obj.children:
                child.disable_updates()
        else:
            self.widget.setUpdatesEnabled(False)

    def set_visible(self, visible):
        """ Show or hide the widget.

        """
        self.widget.setVisible(visible)

    def size_hint(self):
        """ Returns a (width, height) tuple of integers which represent
        the suggested size of the widget for its current state, ignoring
        any windowing decorations. This value is used by the layout 
        manager to determine how much space to allocate the widget.

        """
        if self.use_widget_item_for_layout:
            size_hint = self._widget_item.sizeHint()
        else:
            size_hint = self.widget.sizeHint()
        return Size(size_hint.width(), size_hint.height())

    def layout_geometry(self):
        """ Returns the (x, y, width, height) to of layout geometry
        info for the internal toolkit widget. This should ignore any
        windowing decorations, and may be different than the value
        returned by geometry() if the widget's effective layout rect
        is different from its paintable rect.

        """
        if self.use_widget_item_for_layout:
            geo = self._widget_item.geometry()
        else:
            geo = self.widget.geometry()
        return Rect(geo.x(), geo.y(), geo.width(), geo.height())

    def set_layout_geometry(self, rect):
        """ Sets the layout geometry of the internal widget to the 
        given x, y, width, and height values. The parameters passed
        are equivalent semantics to layout_geometry().

        """
        rect = QRect(*rect)
        if self.use_widget_item_for_layout:
            self._widget_item.setGeometry(rect)
        else:
            self.widget.setGeometry(rect)

    def geometry(self):
        """ Returns an (x, y, width, height) tuple of geometry info
        for the internal toolkit widget, ignoring any windowing
        decorations.

        """
        geom = self.widget.geometry()
        return Rect(geom.x(), geom.y(), geom.width(), geom.height())

    def set_geometry(self, rect):
        """ Sets the geometry of the internal widget to the given
        x, y, width, and height values, ignoring any windowing 
        decorations.

        """
        self.widget.setGeometry(*rect)

    def min_size(self):
        """ Returns the hard minimum (width, height) of the widget, 
        ignoring any windowing decorations. A widget will not be able
        to be resized smaller than this value

        """
        min_size = self.widget.minimumSize()
        return Size(min_size.width(), min_size.height())

    def set_min_size(self, size):
        """ Set the hard minimum width and height of the widget, ignoring
        any windowing decorations. A widget will not be able to be resized 
        smaller than this value.

        """
        self.widget.setMinimumSize(*size)

    def max_size(self):
        """ Returns the hard maximum (width, height) of the widget, 
        ignoring any windowing decorations. A widget will not be able
        to be resized larger than this value

        """
        max_size = self.widget.maximumSize()
        return Size(max_size.width(), max_size.height())

    def set_max_size(self, size):
        """ Set the hard maximum width and height of the widget, ignoring
        any windowing decorations. A widget will not be able to be resized 
        larger than this value.

        """
        # The hard Qt limit is 16777215 (which is 2**24 - 1) and will
        # print warnings to the shell if we attemp to set a max size
        # over that amount. This can be attempted when a QtMainWindow
        # has a central widget size equal to max size, and it also has
        # a menu bar and other components. Clipping the max size like
        # this will not have an effect on layout computation and thus
        # is relatively safe.
        max_width, max_height = size
        max_width = min(max_width, 16777215)
        max_height = min(max_height, 16777215)
        self.widget.setMaximumSize(max_width, max_height)

    def size(self):
        """ Returns the size of the internal toolkit widget, ignoring any
        windowing decorations, as a (width, height) tuple of integers.

        """
        size = self.widget.size()
        return Size(size.width(), size.height())
                
    def resize(self, size):
        """ Resizes the internal toolkit widget according the given
        width and height integers, ignoring any windowing decorations.

        """
        self.widget.resize(*size)

    def pos(self):
        """ Returns the position of the internal toolkit widget as an
        (x, y) tuple of integers, including any windowing decorations. 
        The coordinates should be relative to the origin of the widget's 
        parent, or to the screen if the widget is toplevel.

        """
        pos = self.widget.pos()
        return Pos(pos.x(), pos.y())
            
    def move(self, pos):
        """ Moves the internal toolkit widget according to the given
        x and y integers which are relative to the origin of the
        widget's parent and includes any windowing decorations.

        """
        self.widget.move(*pos)

    def shell_enabled_changed(self, enabled):
        """ The change handler for the 'enabled' attribute on the shell
        object.

        """
        self.set_enabled(enabled)

    def shell_bgcolor_changed(self, color):
        """ The change handler for the 'bgcolor' attribute on the shell
        object. Sets the background color of the internal widget to the 
        given color.
        
        """
        self.set_bgcolor(color)
    
    def shell_fgcolor_changed(self, color):
        """ The change handler for the 'fgcolor' attribute on the shell
        object. Sets the foreground color of the internal widget to the 
        given color.

        """
        self.set_fgcolor(color)

    def shell_font_changed(self, font):
        """ The change handler for the 'font' attribute on the shell 
        object. Sets the font of the internal widget to the given font.

        """
        self.set_font(font)

    def set_enabled(self, enabled):
        """ Enable or disable the widget.

        """
        self.widget.setEnabled(enabled)

    def set_bgcolor(self, color):
        """ Sets the background color of the widget to an appropriate
        QColor given the provided Enaml Color object.

        """
        widget = self.widget
        role = widget.backgroundRole()
        if not color:
            palette = QApplication.instance().palette(widget)
            qcolor = palette.color(role)
            # On OSX, the default color is rendered *slightly* off
            # so a simple workaround is to tell the widget not to
            # auto fill the background.
            widget.setAutoFillBackground(False)
        else:
            qcolor = q_color_from_color(color)
            # When not using qt style sheets to set the background
            # color, we need to tell the widget to auto fill the 
            # background or the bgcolor won't render at all.
            widget.setAutoFillBackground(True)
        palette = widget.palette()
        palette.setColor(role, qcolor)
        widget.setPalette(palette)

    def set_fgcolor(self, color):
        """ Sets the foreground color of the widget to an appropriate
        QColor given the provided Enaml Color object.

        """
        widget = self.widget
        role = widget.foregroundRole()
        if not color:
            palette = QApplication.instance().palette(widget)
            qcolor = palette.color(role)
        else:
            qcolor = q_color_from_color(color)
        palette = widget.palette()
        palette.setColor(role, qcolor)
        widget.setPalette(palette)

    def set_font(self, font):
        """ Sets the font of the widget to an appropriate QFont given 
        the provided Enaml Font object.

        """
        q_font = q_font_from_font(font)
        self.widget.setFont(q_font)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod

from traits.api import Bool, Instance

from .control import Control, AbstractTkControl

from ..noncomponents.abstract_image import AbstractTkImage


class AbstractTkImageView(AbstractTkControl):
    """ The abstract toolkit interface for an ImageView.

    """
    @abstractmethod
    def shell_image_changed(self):
        """ The change handler for the 'image' attribute on the shell 
        component.

        """
        raise NotImplementedError
    
    @abstractmethod
    def shell_scale_to_fit_changed(self):
        """ The change handler for the 'scale_to_fit' attribute on the 
        shell component.

        """
        raise NotImplementedError


class ImageView(Control):
    """ A simple viewer for instances of AbstractTkImage.

    """
    #: A Pixmap instance containing the image to display.
    image = Instance(AbstractTkImage)
    
    #: Whether or not to scale the image with the size of the component.
    scale_to_fit = Bool(False)
    
    #: An image view hugs its width weakly by default.
    hug_width = 'weak'

    #: An image view hugs its height weakly by default.
    hug_height = 'weak'
    
    #: Overridden parent class trait
    abstract_obj = Instance(AbstractTkImageView)




# Copyright (c) 2012 by Enthought, Inc.
# All rights reserved.

from setuptools import setup, find_packages

kwds = {}

try:
    from distutils.core import Extension
    from Cython.Distutils import build_ext

    # only build the speedups Cython C extension when Cython is installed
    kwds['cmdclass'] = {'build_ext': build_ext}
    kwds['ext_modules'] = [Extension('enaml.speedups.model_index',
                                     ['enaml/speedups/model_index.pyx'])]
except ImportError:
    pass


setup(
    name='enaml',
    version='0.2.0',
    author='Enthought, Inc',
    author_email='info@enthought.com',
    url='https://github.com/enthought/enaml',
    description='Declarative language for building dynamic user interfaces '
                'in Python',
    long_description=open('README.rst').read(),
    requires=['traits', 'PySide', 'ply', 'wx', 'casuarius'],
    install_requires=['distribute'],
    packages=find_packages(),
    package_data={'enaml.stdlib': ['*.enaml']},
    entry_points = dict(
        console_scripts = [
            "enaml-run = enaml.runner:main",
        ],
    ),
    test_suite = "enaml.test_collector",
    **kwds
)



#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .wx_constraints_widget import WXConstraintsWidget

from ...components.control import AbstractTkControl


class WXControl(WXConstraintsWidget, AbstractTkControl):
    pass




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from abc import abstractmethod
from datetime import datetime as py_datetime

from traits.api import (
    Property, BaseInstance, Instance, TraitError, on_trait_change,
)

from .control import Control, AbstractTkControl

from ..core.trait_types import Bounded


Datetime = BaseInstance(py_datetime)


class AbstractTkBoundedDatetime(AbstractTkControl):
    """ The abstract toolkit BoundedDatetime.

    """
    @abstractmethod
    def shell_datetime_changed(self, datetime):
        """ The change handler for the 'datetime' attribute on the 
        shell object.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_min_datetime_changed(self, datetime):
        """ The change handler for the 'min_datetime' attribute on the 
        shell object.

        """
        raise NotImplementedError

    @abstractmethod
    def shell_max_datetime_changed(self, datetime):
        """ The change handler for the 'max_datetime' attribute on the 
        shell object.

        """
        raise NotImplementedError


class BoundedDatetime(Control):
    """ A base class for use with widgets that edit a Python 
    datetime.datetime object bounded between minimum and maximum 
    values. This class is not meant to be used directly.

    """
    #: The currently selected datetime. Default value is the current
    #: date and time in the machine. The value is bounded between
    #: :attr:`min_datetime` and :attr:`max_datetime`. Changing the
    #: boundary attributes might result in an update of :attr:`datetime` 
    #: to fit in the new range. Attempts to assign a value outside of 
    #: these bounds will result in a TraitError.
    datetime = Bounded(Datetime(py_datetime.now()),
                       low='min_datetime', high='max_datetime')
                    
    #: The minimum datetime available in the date edit. By default, this
    #: property contains a date that refers to September 14, 1752 and a
    #: time of 00:00:00 and 0 milliseconds. Extra checks take place to
    #: make sure that the user does not programmatically set
    #: :attr:`min_datetime` > :attr:`max_datetime`.
    min_datetime = Property(Datetime, depends_on='_min_datetime')

    #: The internal min datetime storage
    _min_datetime = Datetime(py_datetime(1752,9,14, 0, 0, 0, 0))

    #: The maximum datetime available in the date edit. By default, this
    #: property contains a date that refers to 31 December, 7999 and a
    #: time of 23:59:59 and 999 milliseconds. Extra checks take place to
    #: make sure that the user does not programmatically set
    #: :attr:`min_datetime` > :attr:`max_datetime`.
    max_datetime = Property(Datetime, depends_on='_max_datetime')

    #: The internal max datetime storage
    _max_datetime = Datetime(py_datetime(7999, 12, 31, 23, 59, 59, 999000))

    #: Overridden parent trait
    abstract_obj = Instance(AbstractTkBoundedDatetime)

    #--------------------------------------------------------------------------
    # Properties methods
    #--------------------------------------------------------------------------
    def _set_min_datetime(self, datetime):
        """ Set the min_datetime.

        Addtional checks are perfomed to make sure that
        :attr:`min_datetime` < :attr:`max_datetime`

        """
        if datetime > self.max_datetime:
            msg = ("The minimum datetime of DatetimeEdit should be smaller "
                    "than the current maximum datetime({0}), but a value of "
                    "{1} was given ")
            msg = msg.format(self.max_datetime, datetime)
            raise TraitError(msg)
        self._min_datetime = datetime

    def _set_max_datetime(self, datetime):
        """ Set the max_datetime.

        Addtional checks are perfomed to make sure that
        :attr:`minimum_datetime` < :attr:`maximum_datetime`

        """
        if datetime < self.min_datetime:
            msg = ("The maximum datetime of DatetimeEdit should be larger "
                   "than the current minimum datetime({0}), but a value of "
                   "{1} was given ")
            msg = msg.format(self.min_datetime, datetime)
            raise TraitError(msg)
        self._max_datetime = datetime

    def _get_max_datetime(self):
        """ The property getter for the max datetime.

        """
        return self._max_datetime

    def _get_min_datetime(self):
        """ The property getter for the min datetime.

        """
        return self._min_datetime

    @on_trait_change('min_datetime, max_datetime')
    def _adapt_datetime(self):
        """ Adapt the date to the bounderies

        """
        if self.initialized:
            min_dt, max_dt = self.min_datetime, self.max_datetime
            self.datetime = min(max(self.datetime, min_dt), max_dt)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import datetime

from traits.api import (
    HasTraits, List, Str, Date, Enum, Property, Array, Int, Tuple, 
    on_trait_change, Instance, Event, cached_property, DelegatesTo, Float,
)

import enaml
from enaml.item_models.abstract_item_model import AbstractTableModel
from enaml.styling.color import Color
from enaml.styling.brush import Brush

import stock_data
from plot_driver import PlotDriver

with enaml.imports():
    from stock_view import MainView


class HistoricData(HasTraits):

    available_symbols = List(Str)

    symbol = Enum(values='available_symbols')

    date_range = Tuple(Date, Date)

    end_date = Date

    days_of_history_choices = Property(depends_on='date_range')

    days_of_history = Enum(values='days_of_history_choices')

    start_date = Property(depends_on=['end_date', 'days_of_history'])

    num_points = Int(500)

    data = Array

    value = Float(42)

    def _available_symbols_default(self):
        return stock_data.get_symbols()

    def _date_range_default(self):
        return stock_data.get_symbol_date_range(self.symbol)

    def _end_date_default(self):
        return self.date_range[1] - datetime.timedelta(days=1)

    def _get_days_of_history_choices(self):
        start, end = self.date_range
        return range(100, (end - start).days, 100)

    def _get_start_date(self):
        return self.end_date - datetime.timedelta(days=self.days_of_history)

    def _compute_data(self):
        symbol = self.symbol
        start = self.start_date
        end = self.end_date
        num_points = self.num_points
        return stock_data.get_data(symbol, start, end, num_points)

    def _data_default(self):
        return self._compute_data()

    @on_trait_change('symbol')
    def _refresh_date_range(self, symbol):
        self.date_range = stock_data.get_symbol_date_range(symbol)

    @on_trait_change('symbol, start_date, end_date, num_points')
    def _refresh_data(self):
        self.data = self._compute_data()


class GridDataAdapter(HasTraits):

    model = Instance(HistoricData, ())

    data = DelegatesTo('model')

    available_columns = List(Str)

    grid_columns = List(Str)

    thresh = Property(depends_on='data')

    grid_size = Property(depends_on=['data', 'grid_columns'])

    grid_changed = Event

    def _available_columns_default(self):
        return ['open', 'close', 'low', 'high', 'volume']

    def _grid_columns_default(self):
        return ['open', 'close', 'low', 'high', 'volume']

    @cached_property
    def _get_grid_size(self):
        return (len(self.model.data), len(self.grid_columns))

    @on_trait_change('data, grid_columns')
    def grid_updated(self):
        self.grid_changed = True

    @cached_property
    def _get_thresh(self):
        data = self.data
        min = data['close'].min()
        max = data['close'].max()
        return 0.9 * (max - min) + min


HIGHLIGHT = Brush(Color.from_string('lightskyblue'), None)

TEXTCOLOR = Brush(Color.from_string('darkgray'), None)

NULLBRUSH = None


class StockDataTable(AbstractTableModel):

    def __init__(self, adapter):
        super(StockDataTable, self).__init__()
        self.adapter = adapter
        adapter.on_trait_change(self.refresh_table, 'grid_changed')

    def refresh_table(self):
        self.begin_reset_model()
        self.end_reset_model()

    def column_count(self, parent=None):
        if parent is None:
            return self.adapter.grid_size[1]
        return 0

    def row_count(self, parent=None):
        if parent is None:
            return self.adapter.grid_size[0]
        return 0
        
    def data(self, index):
        adapter = self.adapter
        data = adapter.data
        column = adapter.grid_columns[index.column]
        data = data[column][index.row]
        if data > 1e4:
            res = '%.2E' % data
        else:
            res = '%.2f' % data
        return res
    
    def background(self, index):
        adapter = self.adapter
        data = adapter.data['close'][index.row]
        if data > adapter.thresh:
            return HIGHLIGHT
        return NULLBRUSH
    
    def foreground(self, index):
        adapter = self.adapter
        data = adapter.data['close'][index.row]
        if data > adapter.thresh:
            return NULLBRUSH
        return TEXTCOLOR

    def vertical_header_data(self, section):
        data = self.adapter.data
        ts = data['dates'][section]
        return str(datetime.date.fromtimestamp(ts))
    
    def horizontal_header_data(self, section):
        return self.adapter.grid_columns[section].capitalize()


if __name__ == '__main__':
    model = HistoricData()
    plot_driver = PlotDriver(model)
    adapter = GridDataAdapter(model=model)
    data_table = StockDataTable(adapter)

    view = MainView(model=model, adapter=adapter, plot=plot_driver,
                    stock_data_table=data_table)
    view.show()




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt import QtGui
from .qt_widget_component import QtWidgetComponent

from ...components.menu_bar import AbstractTkMenuBar


class QtMenuBar(QtWidgetComponent, AbstractTkMenuBar):
    """ A Qt4 implementation of a MenuBar.

    """
    def create(self, parent):
        """ Creates the underlying QMenuBar.

        """
        # We ignore the parent when creating the menu bar. The parent 
        # main window is repsonsible for setting the menu bar on itself, 
        # and using the parent here causes issues on certain platforms.
        self.widget = QtGui.QMenuBar()

    def initialize(self):
        """ Initializes the QMenuBar.

        """
        super(QtMenuBar, self).initialize()
        self.update_menus()

    #--------------------------------------------------------------------------
    # Change Handlers 
    #--------------------------------------------------------------------------
    def shell_menus_changed(self, menus):
        """ Update the menu bar with the new list of menu objects.

        """
        self.update_menus()

    #--------------------------------------------------------------------------
    # Widget Update Methods 
    #--------------------------------------------------------------------------
    def update_menus(self):
        """ Updates the menu bar with the child menu objects.

        """
        # There is no need to clear() the Menu since that destroys the
        # underlying objects, and any dynamic children will have already
        # been destroyed. It's sufficient to just make a pass over the 
        # menus and add them to the menu bar. Qt is smart enough to do 
        # the right thing here.
        widget = self.widget
        for menu in self.shell_obj.menus:
            widget.addMenu(menu.toolkit_widget)




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from ast import literal_eval
from numbers import Number, Real, Integral, Complex

from .coercing_validator import CoercingValidator


#------------------------------------------------------------------------------
# Base Number Validator
#------------------------------------------------------------------------------
class NumberValidator(CoercingValidator):
    """ A CoercingValidator subclass which accepts number strings which 
    will be converted using the builtin Python number types.

    """
    #: The type of the number to which the string must evaluate to be
    #: considered for validity.
    number_type = Number

    def __init__(self, low=None, high=None, precision=-1, **kwargs):
        """ Initialize a NumberValidator.

        Parameters
        ----------
        low : Number or None, optional
            The lowest acceptable value for the number. If None, there
            is no lower limit to the value. The default is None.
        
        high : Number or None, optional
            The highest acceptable value for the number. If None, there
            is no upper limit to the value. The default is None.

        precision : integer, optional
            An optional integer to control the precision of floating
            point number formatting. The default is -1 and indicates
            that implementation default precision should be used.

        **kwargs
            The keyword arguments necessary to initialize an instance
            of CoercingValidator.

        """
        super(NumberValidator, self).__init__(**kwargs)
        self.low = low
        self.high = high
        self.precision = precision

    def _convert(self, text):
        """ Overridden default convert method which uses the function
        ast.literal_eval to safetly evaluate the text. 
        
        Subclasses may override this to provide locale-aware text 
        conversion.

        """
        return literal_eval(text)

    def _post_convert_fail(self, text):
        """ A method intended to be overridden by subclasses which is
        called when conversion of the text fails. Such a failure would
        normally consider the input as INVALID. However, if this method
        returns True, then the text will be considered INTERMEDIATE. This
        is useful to handle cases where, for example, scientific notation
        is being entered and the trailing [eE][+-]? characters cause
        conversion to fail.

        Returns
        -------
        result : bool
            True if the failing text should be considered INTERMEDIATE,
            False otherwise.

        """
        return False

    def validate(self, text):
        """ Validates the input text against the rules of the validator.

        Parameters
        ----------
        text : unicode string
            The input string supplied by the user.
        
        Returns
        -------
        result : int
            One of the class attribute constants INVALID, INTERMEDIATE, 
            or VALID.
        
        Notes
        -----
        The string is INVALID under the following conditions:
            - The string fails coercing validation.
        
        The string is INTERMEDIATE under the following conditions:
            - The string is empty.
            - The coerced value is less than the low value.
            - The coerced value is greater than the high value.

        The string is ACCEPTABLE for all other situations.

        """
        if not text:
            return self.INTERMEDIATE

        validity, value = self._validate(text)
        if validity == self.INVALID:
            if self._post_convert_fail(text):
                validity = self.INTERMEDIATE
            return validity
            
        if not isinstance(value, self.number_type):
            return self.INVALID

        low = self.low
        if low is not None:
            if value < low:
                if value < 0:
                    return self.INVALID
                else:
                    return self.INTERMEDIATE
        
        high = self.high
        if high is not None:
            if value > high:
                if value > 0:
                    return self.INVALID
                else:
                    return self.INTERMEDIATE
        
        return self.ACCEPTABLE


class _PostConvertMixin(object):
    """ A validator mixin class which provides a post convert fail method
    which validates failing text based on given endings.

    """
    valid_fail_endings = ()

    def _post_convert_fail(self, text):
        """ Validates failing text based on the endings provided in the
        'valid_fail_endings' iterable.

        """
        for ending in self.valid_fail_endings:
            if text == ending:
                return True
            if text.endswith(ending):
                new = text[:-len(ending)]
                validity, value = self._validate(new)
                if validity != self.INVALID:
                    return True
        return False


#------------------------------------------------------------------------------
# numbers.Number Validators
#------------------------------------------------------------------------------
class IntegralNumberValidator(NumberValidator):
    """ A NumberValidator for Integral number types using the abstract 
    base class numbers.Integral for type checking.

    """
    number_type = Integral


class RealNumberValidator(_PostConvertMixin, NumberValidator):
    """ A NumberValidator for Real number types using the abstract 
    base class numbers.Real for type checking.

    """
    number_type = Real

    valid_fail_endings = (u'-', u'e', u'E', u'e+', u'E+', u'e-', u'E-')


class ComplexNumberValidator(NumberValidator):
    """ A NumberValidator for Complex number types using the abstract 
    base class numbers.Complex for type checking.

    """
    number_type = Complex

    def _post_convert_fail(self, text):
        """ A post convert fail method which allows trailing '+' and
        '-' symbols, as well scientific notation and missing 'j' 
        characters.

        """
        if text == u'-':
            return True

        if text.endswith(u'+') and text != u'+':
            new = text[:-1]
        elif u'+' in text and not text.endswith(u'j'):
            new = text + u'j'
        else:
            new = text
        
        validity, value = self._validate(new)
        if validity != self.INVALID:
            return True
        
        # If the text is still invalid, we may be entering trailing
        # scientific notation which we explicitly allow. Rather than 
        # ignoring the symbols, we attempt to complete them by adding
        # a 0 exponential. This makes the substitution position aware.
        for sub in (u'e+', u'e-', u'E+', u'E-', u'e', u'E'):
            subbed = new.replace(sub, sub + u'0')
            validity, value = self._validate(subbed)
            if validity != self.INVALID:
                return True
        
        return False


#------------------------------------------------------------------------------
# Concrete Number Validators
#------------------------------------------------------------------------------
class IntValidator(IntegralNumberValidator):
    """ A NumberValidator for 'int' number types which uses the given
    locale object to perform conversion.

    """
    number_type = int

    def _format(self, value):
        """ Overridden default format method which uses the provided
        locale object to perform the formatting.

        """
        return self.locale.from_int(value, group=True)

    def _convert(self, text):
        """ Overridden default convert method which uses the provided
        locale object to perform the conversion.

        """
        return self.locale.to_int(text)


class LongValidator(IntegralNumberValidator):
    """ A NumberValidator for 'long' number types which uses the given
    locale object to perform conversion.

    """
    number_type = long

    def _format(self, value):
        """ Overridden default format method which uses the provided
        locale object to perform the formatting.

        """
        return self.locale.from_long(value, group=True)

    def _convert(self, text):
        """ Overridden default convert method which uses the provided
        locale object to perform the conversion.

        """
        return self.locale.to_long(text)


class FloatValidator(RealNumberValidator):
    """ A NumberValidator for 'float' number types which uses the given
    locale object to perform conversion.

    """
    number_type = float

    def _format(self, value):
        """ Overridden default format method which uses the provided
        locale object to perform the formatting.

        """
        return self.locale.from_float(value, group=True, prec=self.precision)

    def _convert(self, text):
        """ Overridden default convert method which uses the provided
        locale object to perform the conversion.

        """
        return self.locale.to_float(text)


class ComplexValidator(ComplexNumberValidator):
    """ A NumberValidator for 'complex' number types which uses the 
    given locale object to perform conversion.

    """
    number_type = complex

    def _format(self, value):
        """ Overridden default format method which uses the provided
        locale object to perform the formatting.

        """
        return self.locale.from_complex(value, prec=self.precision)

    def _convert(self, text):
        """ Overridden default convert method which uses the provided
        locale object to perform the conversion.

        """
        return self.locale.to_complex(text)


class BinValidator(IntValidator):
    """ An IntValidator which converts and displays as a binary string
    and uses the given locale object to perform conversion.

    """
    def _format(self, value):
        """ Overridden default format method which uses the provided
        locale object to perform the formatting.

        """
        return self.locale.from_int(value, base=2)

    def _convert(self, text):
        """ Overridden default convert method which uses the provided
        locale object to perform the conversion.

        """
        return self.locale.to_int(text, base=2)


class OctValidator(IntValidator):
    """ An IntValidator which converts and displays as an octal string
    and uses the given locale object to perform conversion.

    """
    def _format(self, value):
        """ Overridden default format method which uses the provided
        locale object to perform the formatting.

        """
        return self.locale.from_int(value, base=8)

    def _convert(self, text):
        """ Overridden default convert method which uses the provided
        locale object to perform the conversion.

        """
        return self.locale.to_int(text, base=8)


class HexValidator(_PostConvertMixin, IntValidator):
    """ An IntValidator which converts and displays as a hex string
    and uses the given locale object to perform conversion.

    """
    valid_fail_endings = (u'0x', u'0X')

    def _format(self, value):
        """ Overridden default format method which uses the provided
        locale object to perform the formatting.

        """
        return self.locale.from_int(value, base=16)

    def _convert(self, text):
        """ Overridden default convert method which uses the provided
        locale object to perform the conversion.

        """
        return self.locale.to_int(text, base=16)


class LongBinValidator(LongValidator):
    """ A LongValidator which converts and displays as a binary string
    and uses the given locale object to perform conversion.

    """
    def _format(self, value):
        """ Overridden default format method which uses the provided
        locale object to perform the formatting.

        """
        return self.locale.from_long(value, base=2)

    def _convert(self, text):
        """ Overridden default converts method which uses the provided
        locale object to perform the conversion.

        """
        return self.locale.to_long(text, base=2)


class LongOctValidator(LongValidator):
    """ A LongValidator which converts and displays as an octal string
    and uses the given locale object to perform conversion.

    """
    def _format(self, value):
        """ Overridden default format method which uses the provided
        locale object to perform the formatting.

        """
        return self.locale.from_long(value, base=8)

    def _convert(self, text):
        """ Overridden default convert method which uses the provided
        locale object to perform the conversion.

        """
        return self.locale.to_long(text, base=8)


class LongHexValidator(_PostConvertMixin, LongValidator):
    """ A LongValidator which converts and displays as a hex string
    and uses the given locale object to perform conversion.

    """
    valid_fail_endings = (u'0x', u'0X')

    def _format(self, value):
        """ Overridden default format method which uses the provided
        locale object to perform the formatting.

        """
        return self.locale.from_long(value, base=16)

    def _convert(self, text):
        """ Overridden default convert method which uses the provided
        locale object to perform the conversion.

        """
        return self.locale.to_long(text, base=16)




# -*- coding: UTF-8 -*-
#------------------------------------------------------------------------------
#  file: class_doc.py
#  License: LICENSE.TXT
#  Author: Ioannis Tziakos
#
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from base_doc import BaseDoc
from line_functions import get_indent, replace_at, add_indent
from fields import (max_header_length, max_desc_length,
                    max_name_length, MethodField, AttributeField)


class ClassDoc(BaseDoc):
    """Docstring refactoring for classes"""

    def __init__(self, lines, headers=None, verbose=False):

        if headers is None:
            headers = {'Attributes': 'attributes', 'Methods': 'methods',
                       'See Also': 'header', 'Abstract Methods': 'methods',
                       'Notes':'notes'}

        super(ClassDoc, self).__init__(lines, headers, verbose)
        return

    def _refactor_attributes(self, header):
        """Refactor the attributes section to sphinx friendly format"""

        if self.verbose:
            print '{0} Section'.format(header)

        index = self.index
        self.remove_lines(index, 2)
        indent = get_indent(self.peek())
        fields = self.extract_fields(indent, AttributeField)

        descriptions = []
        for field in fields:
            descriptions += field.to_rst(len(indent))
        self.insert_lines(descriptions[:-1], index)
        self.index += len(descriptions)
        return

    def _refactor_methods(self, header):
        """Refactor the methods section to sphinx friendly format.

        """
        if self.verbose:
            print '{0} section'.format(header)

        index = self.index
        self.remove_lines(index, 2)
        indent = get_indent(self.peek())
        method_fields = self.extract_fields(indent, MethodField)

        lines = []
        if len(method_fields) > 0 :
            name_length = max_name_length(method_fields)
            method_length = max_header_length(method_fields)
            desc_length = max_desc_length(method_fields)

            first_column = len(indent)
            second_column = first_column + method_length + name_length + 13
            first_column_str = '=' * (method_length + name_length + 12)
            second_column_str = '=' * desc_length

            border = '{0}{1} {2}'.format(indent,
                                              first_column_str,
                                              second_column_str)
            length = len(border)
            empty = length * ' '
            headings = empty[:]
            headings = replace_at('Methods', headings, first_column)
            headings = replace_at('Description', headings, second_column)
            lines.append(border)
            lines.append(headings)
            lines.append(border)
            for field in method_fields:
                lines += field.to_rst(length, first_column, second_column)
            lines.append(border)

        lines = [line.rstrip() for line in lines]
        self.insert_lines(lines, index)
        self.index += len(lines)
        return

    def _refactor_notes(self, header):
        """Refactor the note section to use the rst ``.. note`` directive.

        """
        descriptions = []
        index = self.index
        self.remove_lines(index, 2)
        indent = get_indent(self.peek())
        paragraph = self.get_next_paragraph()
        descriptions.append(indent + '.. note::')
        descriptions += add_indent(paragraph)
        self.insert_lines(descriptions, index)
        self.index += len(descriptions)
        return descriptions



#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from traits.api import (
    List, Instance, Property, cached_property, Bool, WeakRef
)

from .constraints_widget import (
    ConstraintsWidget, AbstractTkConstraintsWidget,
)
from .layout_task_handler import LayoutTaskHandler

from ..layout.constrainable import PaddingConstraints, Constrainable
from ..layout.constraints_layout import ConstraintsLayout
from ..layout.layout_helpers import expand_constraints
from ..layout.geometry import Size, Box
    

class AbstractTkContainer(AbstractTkConstraintsWidget):
    """ The abstract toolkit Container interface.

    """
    pass


class Container(LayoutTaskHandler, PaddingConstraints, ConstraintsWidget):
    """ A Component subclass that provides functionality for laying out 
    constrainable children according to their system of constraints.

    """
    #: A read-only cached property which returns the constraints layout
    #: manager for this container, or None if the layout is being managed
    #: by a parent container.
    layout_manager = Property(
        Instance(ConstraintsLayout), depends_on='owns_layout',
    )

    #: The list of children that can participate in constraints based
    #: layout. This list is composed of components in the list of 
    #: children that are instances of Constrainable.
    constraints_children = Property(
        List(Instance(Constrainable)), depends_on='children',
    )

    #: A read-only property which returns True if this container owns
    #: its layout and is responsible for setting the geometry of its
    #: children, or False if that responsibility has been transferred
    #: to another component in the hierarchy.
    owns_layout = Property(Bool, depends_on='_layout_owner')

    #: A container has a default padding of 10 on all sides.
    padding = Box(10, 10, 10, 10)
    
    #: A private trait which stores a weak reference to the owner of 
    #: the layout for this container, or None if this container owns 
    #: its layout.
    _layout_owner = WeakRef(allow_none=True)

    #: A private cached property which computes the size hint whenever 
    #: the size_hint_updated event is fired.
    _size_hint = Property(Instance(Size), depends_on='size_hint_updated')

    #: Overridden parent class trait.
    abstract_obj = Instance(AbstractTkContainer)

    #--------------------------------------------------------------------------
    # Property Getters
    #--------------------------------------------------------------------------
    @cached_property
    def _get_constraints_children(self):
        """ Cached property getter for the 'constraints_children' 
        attribute. This getter returns the sublist of children that are 
        instances of ConstraintsWidget.

        """
        flt = lambda child: isinstance(child, Constrainable)
        return filter(flt, self.children)
    
    @cached_property
    def _get_layout_manager(self):
        """ The property getter for the 'layout_manager' attribute.

        """
        if self.owns_layout:
            return ConstraintsLayout()

    def _get_owns_layout(self):
        """ The property getter for the 'owns_layout' attribute.

        """
        return self._layout_owner is None

    @cached_property
    def _get__size_hint(self):
        """ The property getter for the '_size_hint' attribtue.

        """
        # XXX we can probably do better than the min size. Maybe have 
        # the layout manager compute a preferred size, or something
        # similar to a preferred size. But I don't know at the moment
        # what it would actually mean to have a preferred size from 
        # a set of constraints.
        return self.compute_min_size()

    #--------------------------------------------------------------------------
    # Change Handlers
    #--------------------------------------------------------------------------
    def _constraints_children_changed(self):
        """ A handler which requests a relayout when the constraints
        children change, provided that the container is initialized.

        """
        if self.initialized:
            self.request_relayout()

    #--------------------------------------------------------------------------
    # Setup Methods 
    #--------------------------------------------------------------------------
    def _setup_init_layout(self):
        """ A reimplemented parent class setup method that performs any
        layout initialization necessary for the component. The layout is
        initialized from the bottom up.

        """
        super(Container, self)._setup_init_layout()
        self.initialize_layout()

    #--------------------------------------------------------------------------
    # Layout Handling
    #--------------------------------------------------------------------------
    def initialize_layout(self):
        """ A reimplemented parent class method that initializes the 
        layout manager for the first time, and binds the relevant
        layout change handlers.

        """
        # We only need to initialize the manager if we own the layout.
        if self.owns_layout:
            constraints = self.compute_constraints()
            self.layout_manager.initialize(constraints)
            # We fire off a size hint updated event here since, if
            # for some reason, the size hint was computed before 
            # the layout was initialized, the value will be wrong
            # and cached. So we need to clear the cache so that it 
            # will be recomputed by the next consumer that needs it.
            self.size_hint_updated()

    def relayout(self):
        """ A reimplemented parent class method which forwards the call
        to the layout owner if necessary.

        """
        if self.owns_layout:
            super(Container, self).relayout()
        else:
            self._layout_owner.relayout()
            
    def refresh(self):
        """ A reimplemented parent class method which forwards the call
        to the layout owner if necessary.

        """
        if self.owns_layout:
            super(Container, self).refresh()
        else:
            self._layout_owner.refresh()

    def request_relayout(self):
        """ A reimplemented parent class method which forwards the call
        to the layout owner if necessary.

        """
        if self.owns_layout:
            super(Container, self).request_relayout()
        else:
            self._layout_owner.request_relayout()

    def request_refresh(self):
        """ A reimplemented parent class method which forwards the call
        to the layout owner if necessary.

        """
        if self.owns_layout:
            super(Container, self).request_refresh()
        else:
            self._layout_owner.request_refresh()

    def request_relayout_task(self, callback, *args, **kwargs):
        """ A reimplemented parent class method which forwards the call
        to the layout owner if necessary.

        """
        if self.owns_layout:
            sup = super(Container, self)
            sup.request_relayout_task(callback, *args, **kwargs)
        else:
            self._layout_owner.request_relayout_task(callback, *args, **kwargs)
            
    def request_refresh_task(self, callback, *args, **kwargs):
        """ A reimplemented parent class method which forwards the call
        to the layout owner if necessary.

        """
        if self.owns_layout:
            sup = super(Container, self)
            sup.request_refresh_task(callback, *args, **kwargs)
        else:
            self._layout_owner.request_refresh_task(callback, *args, **kwargs)
        
    def do_relayout(self):
        """ A reimplemented LayoutTaskHandler handler method which will
        actually perform the layout.

        """
        # At this point, we know that we own the layout since the
        # calls that trigger the call to this method would have 
        # already been forwarded on to the layout owner. So, at
        # this point, we just have to recompute the constraints
        # and do a refresh.

        # TODO There is a big opportunity here to optimize by calling
        # .update_constraints() on the layout manager instead of just
        # recomputing everything from scratch, but that will require
        # tracking the created constraints so for now we just punt.
        self.layout_manager.initialize(self.compute_constraints())
        self.do_refresh()

        # We emit the size hint updated event at this point since
        # we are still inside a freeze context. This means that if
        # our parent is a toplevel window, it can set the new size
        # of the window before leaving the context which helps 
        # eleminate flicker during the resize.
        self.size_hint_updated()

    def do_refresh(self):
        """ A reimplemented LayoutTaskHandler handler method which will
        actually perform the refresh.

        """
        # At this point, we know that we own the layout since the
        # calls that trigger the call to this method would have 
        # already been forwarded on to the layout owner. So, at
        # this point, we just have to do a refresh.
        width = self.width
        height = self.height
        size = self.size()
        self.layout_manager.layout(self.apply_layout, width, height, size)

    def apply_layout(self):
        """ The callback invoked by the layout manager when there are
        new layout values available. This traverses the constraints 
        children for which this container has layout ownership and 
        applies the geometry updates.

        """
        stack = [((0, 0), self.constraints_children)]
        pop = stack.pop
        push = stack.append
        while stack:
            offset, children = pop()
            for child in children:
                new_offset = child.update_layout_geometry(*offset)
                if isinstance(child, Container):
                    if child._layout_owner is self:
                        push((new_offset, child.constraints_children))

    #--------------------------------------------------------------------------
    # Constraints Computation
    #--------------------------------------------------------------------------
    def compute_constraints(self):
        """ Descends the tree for all containers and children for which
        this container can manage layout, and aggregates all of their
        constraints into a single list.

        """
        expand = expand_constraints

        cns = []
        cns_extend = cns.extend

        # We don't care about the size hint constraints for a container
        # which manages a layout because the actual size is the input
        # to the solver.
        cns_extend(expand(self, self.hard_constraints()))
        cns_extend(expand(self, self.padding_constraints()))
        cns_extend(expand(self, self.user_constraints()))
        cns_extend(expand(self, self.component_constraints()))

        stack = list(self.constraints_children)
        stack_pop = stack.pop
        stack_extend = stack.extend
        while stack:
            child = stack_pop()
            if isinstance(child, Container):
                # When we take over layout ownership of a container we
                # don't care about its size hint constraints since we
                # deal with its children directly.
                if child.transfer_layout_ownership(self):
                    cns_extend(expand(child, child.hard_constraints()))
                    cns_extend(expand(child, child.padding_constraints()))
                    cns_extend(expand(child, child.user_constraints()))
                    cns_extend(expand(child, child.component_constraints()))
                    stack_extend(child.constraints_children)
                else:
                    # If we aren't taking over layout ownership, then we
                    # don't care about any of the container's internal 
                    # constraints.
                    cns_extend(expand(child, child.hard_constraints()))
                    cns_extend(expand(child, child.size_hint_constraints()))
            else:
                cns_extend(expand(child, child.hard_constraints()))
                cns_extend(expand(child, child.size_hint_constraints()))
                cns_extend(expand(child, child.user_constraints()))
                cns_extend(expand(child, child.component_constraints()))
                if isinstance(child, PaddingConstraints):
                    cns_extend(expand(child, child.padding_constraints()))
                    
        return cns

    def default_user_constraints(self):
        """ Constraints to use if the constraints trait is an empty list.
        The default container behavior is to put the layout children into
        a vertical box layout.

        """
        from ..layout.layout_helpers import vbox
        return [vbox(*self.constraints_children)]

    #--------------------------------------------------------------------------
    # Overrides
    #--------------------------------------------------------------------------
    def size_hint(self):
        """ Overridden parent class method to return the size hint of 
        the container from the layout manager. If the container does not
        own its layout, or if the layout has not been initialized, then
        this method will return (-1, -1). This method does not rely on 
        the size hint computation of the underlying toolkit widget. Thus,
        the underlying widget may call back into this method as needed
        to get a size hint from the layout manager.

        """
        # Since this may be called very often by user code, especially 
        # if the toolkit widget is using it as a replacement for its
        # internal size hint computation, we must cache the value or
        # it will be too expensive to use under heavy resize loads.
        # This returns the value from the cached property which is 
        # updated whenver the size_hint_updated event is fired.
        return self._size_hint

    #--------------------------------------------------------------------------
    # Auxiliary Methods
    #--------------------------------------------------------------------------
    def transfer_layout_ownership(self, owner):
        """ A method which can be called by other components in the
        hierarchy to gain ownership responsibility for the layout 
        of the children of this container. By default, the transfer
        is allowed and is the mechanism which allows constraints to
        cross widget boundaries. Subclasses should reimplement this 
        method if different behavior is desired.

        Parameters
        ----------
        owner : BaseComponent
            The component which has taken ownership responsibility
            for laying out the children of this component. All 
            relayout and refresh requests will be forwarded to this
            component.
        
        Returns
        -------
        results : bool
            True if the transfer was allowed, False otherwise.
        
        """
        self._layout_owner = owner
        return True
    
    def compute_min_size(self):
        """ Calculates the minimum size of the container which would 
        allow all constraints to be satisfied. If this container does
        not own its layout, or if the layout has not been initialized,
        then this method will return (-1, -1).

        """
        if self.owns_layout and self.layout_manager.initialized:
            width = self.width
            height = self.height
            w, h = self.layout_manager.get_min_size(width, height)
            res = Size(int(round(w)), int(round(h)))
        else:
            res = Size(-1, -1)
        return res

    def compute_max_size(self):
        """ Calculates the maximum size of the container which would 
        allow all constraints to be satisfied. If this container does
        not own its layout, or if the layout has not been initialized
        then this method will return (-1, -1).

        """
        if self.owns_layout and self.layout_manager.initialized:
            width = self.width
            height = self.height
            w, h = self.layout_manager.get_max_size(width, height)
            res = Size(int(round(w)), int(round(h)))
        else:
            res = Size(-1, -1)
        return res




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
from .qt import QtGui, QtCore
from .qt_control import QtControl

from ...components.slider import AbstractTkSlider


#: A map from Enaml constants to QSlider TickPosition values.
TICK_POS_MAP = {'no_ticks': QtGui.QSlider.NoTicks,
                'left': QtGui.QSlider.TicksLeft,
                'right': QtGui.QSlider.TicksRight,
                'top': QtGui.QSlider.TicksAbove,
                'bottom': QtGui.QSlider.TicksBelow,
                'both': QtGui.QSlider.TicksBothSides}


#: A map from Enaml enums to Qt constants for horizontal or vertical 
#: orientation.
ORIENTATION_MAP = {'horizontal': QtCore.Qt.Horizontal,
                   'vertical': QtCore.Qt.Vertical}


# Qt Slider does not return TicksLeft and TicksRight it always
# converts these to TicksAbove and TicksBelow so we need to check
# the orientation in order to return the right result

#: A map from horizontal QSlider TickPosition values to Enaml 
#: constants.
HOR_TICK_POS_MAP = {QtGui.QSlider.NoTicks: 'no_ticks',
                    QtGui.QSlider.TicksAbove: 'top',
                    QtGui.QSlider.TicksBelow: 'bottom',
                    QtGui.QSlider.TicksBothSides: 'both'}


#: A map from vertical QSlider TickPosition values to Enaml 
#: constants.
VERT_TICK_POS_MAP = {QtGui.QSlider.NoTicks: 'no_ticks',
                    QtGui.QSlider.TicksAbove: 'left',
                    QtGui.QSlider.TicksBelow: 'right',
                    QtGui.QSlider.TicksBothSides: 'both'}


class QtSlider(QtControl, AbstractTkSlider):
    """ A Qt implementation of Slider.

    """
    #--------------------------------------------------------------------------
    # Setup Methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying QSlider widget.

        """
        self.widget = QtGui.QSlider(parent=parent)

    def initialize(self):
        """ Initializes the attributes of the toolkit widget.

        """
        super(QtSlider, self).initialize()
        shell = self.shell_obj
        shell._down = False
        self.set_range(shell.minimum, shell.maximum)
        self.set_position(shell.value)
        self.set_tick_position(shell.tick_position)
        self.set_orientation(shell.orientation)
        self.set_tick_frequency(shell.tick_interval)
        self.set_single_step(shell.single_step)
        self.set_page_step(shell.page_step)
        self.set_tracking(shell.tracking)

    def bind(self):
        """ Connect the event handlers for the slider widget signals.

        """
        super(QtSlider, self).bind()
        widget = self.widget
        widget.valueChanged.connect(self._on_slider_changed)
        widget.sliderMoved.connect(self._on_slider_moved)
        widget.sliderPressed.connect(self._on_pressed)
        widget.sliderReleased.connect(self._on_released)

    #--------------------------------------------------------------------------
    # Implementation
    #--------------------------------------------------------------------------
    def shell_minimum_changed(self, minimum):
        """ The change handler for the 'minimum' attribute on the shell
        object.

        """
        self.set_range(minimum, self.shell_obj.maximum)

    def shell_maximum_changed(self, maximum):
        """ The change handler for the 'maximum' attribute on the shell
        object.
        
        """
        self.set_range(self.shell_obj.minimum, maximum)

    def shell_value_changed(self, value):
        """ The change handler for the 'value' attribute on the shell
        object.
        
        """
        self.set_position(value)

    def shell_tracking_changed(self, tracking):
        """ The change handler for the 'tracking' attribute on the shell
        object.

        """
        self.set_tracking(tracking)

    def shell_single_step_changed(self, single_step):
        """ The change handler for the 'single_step' attribute on the 
        shell object.
        
        """
        self.set_single_step(single_step)

    def shell_page_step_changed(self, page_step):
        """ The change handler for the 'page_step' attribute on the 
        shell object.

        """
        self.set_page_step(page_step)

    def shell_tick_interval_changed(self, tick_interval):
        """ The change handler for the 'tick_interval' attribute on the
        shell object.

        """
        shell = self.shell_obj
        self.set_tick_frequency(tick_interval)
        # This extra calls are made since the range trait on 
        # shell object may clip the values to fit within the 
        # range without firing a changed notification.
        self.set_single_step(shell.single_step)
        self.set_page_step(shell.page_step)

    def shell_tick_position_changed(self, tick_position):
        """ The change handler for the 'tick_position' attribute on the
        shell object.

        """
        self.set_tick_position(tick_position)
        self.shell_obj.size_hint_updated()
        
    def shell_orientation_changed(self, orientation):
        """ The change handler for the 'orientation' attribute on the 
        shell object.

        """
        self.set_orientation(orientation)
        self.shell_obj.size_hint_updated()

    #--------------------------------------------------------------------------
    # Event Handlers 
    #--------------------------------------------------------------------------
    def _on_slider_changed(self):
        """ The event handler for the slider value changed event.

        """
        self.shell_obj.value = self.widget.value()

    def _on_slider_moved(self):
        """ The event handler for a slider moved event.

        """
        self.shell_obj.moved(self.widget.sliderPosition())

    def _on_pressed(self):
        """ Update if the left button was pressed.

        Estimates the position of the thumb and then checks if the mouse
        was pressed over it to fire the `pressed` event and sets the
        `down` attribute.

        """
        shell = self.shell_obj
        shell._down = True
        shell.pressed()

    def _on_released(self):
        """ Update if the left button was released

        Checks if the `down` attribute was set. In that case the
        function fires the release event and sets the `down` attribute to
        false.

        """
        shell = self.shell_obj
        if shell._down:
            shell._down = False
            shell.released()

    #--------------------------------------------------------------------------
    # Widget Update Methods 
    #--------------------------------------------------------------------------
    def set_single_step(self, step):
        """ Set the single step attribute in the Qt widget.

        """
        self.widget.setSingleStep(step)

    def set_page_step(self, step):
        """ Set the page step attribute in the Qt widget.

        """
        self.widget.setPageStep(step)

    def set_tick_position(self, ticks):
        """ Apply the tick position in the widget.

        """
        self.widget.setTickPosition(TICK_POS_MAP[ticks])
        self.sync_tick_position()

    def set_orientation(self, orientation):
        """ Set the slider orientation.

        """
        self.widget.setOrientation(ORIENTATION_MAP[orientation])
        self.sync_tick_position()

    def set_tracking(self, tracking):
        """ Set the tracking state of the slider.

        """
        self.widget.setTracking(tracking)

    def set_range(self, minimum, maximum):
        """ Set the range of the slider widget.

        """
        self.widget.setRange(minimum, maximum)

    def set_tick_frequency(self, interval):
        """ Set the slider widget tick mark fequency.

        """
        self.widget.setTickInterval(interval)

    def set_position(self, value):
        """ Validate the position value.

        """
        self.widget.setValue(value)

    def sync_tick_position(self):
        """ Syncornize the tick_position with the widget. The QSlider 
        automatically updates or adaptes the tick position but we still 
        need to update the enaml component, so that it is in sync.

        """
        shell = self.shell_obj
        orientation = shell.orientation
        tick_pos = self.widget.tickPosition()
        if orientation == 'vertical':
            shell.tick_position = VERT_TICK_POS_MAP[tick_pos]
        else:
            shell.tick_position = HOR_TICK_POS_MAP[tick_pos]




#------------------------------------------------------------------------------
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
#------------------------------------------------------------------------------
import datetime

import wx

from .wx_bounded_date import WXBoundedDate

from ...components.date_edit import AbstractTkDateEdit


def to_wx_date(py_date):
    day = py_date.day
    month = py_date.month - 1  # wx peculiarity!
    year = py_date.year
    return wx.DateTimeFromDMY(day, month, year)


def from_wx_date(wx_date):
    if wx_date.IsValid():
        day = wx_date.GetDay()
        month = wx_date.GetMonth() + 1  # wx peculiarity!
        year = wx_date.GetYear()
        return datetime.date(year, month, day)


class WXDateEdit(WXBoundedDate, AbstractTkDateEdit):
    """ A WX implementation of DateEdit.

    """
    #--------------------------------------------------------------------------
    # Setup methods
    #--------------------------------------------------------------------------
    def create(self, parent):
        """ Creates the underlying wx.DatePickerCtrl.

        """
        self.widget = wx.DatePickerCtrl(parent)

    def initialize(self):
        """ Initializes the attributes of the control.

        """
        super(WXDateEdit, self).initialize()
        self.set_format(self.shell_obj.date_format)

    def bind(self):
        """ Binds the event handlers for the date widget.

        """
        super(WXDateEdit, self).bind()
        self.widget.Bind(wx.EVT_DATE_CHANGED, self.on_date_changed)

    #--------------------------------------------------------------------------
    # Component attribute notifiers
    #--------------------------------------------------------------------------
    def shell_date_format_changed(self, date_format):
        """ The change handler for the 'format' attribute.

        """
        self.set_format(date_format)

    #--------------------------------------------------------------------------
    # Signal handlers
    #--------------------------------------------------------------------------
    def on_date_changed(self, event):
        """ The event handler for the date's changed event.

        """
        new_date = from_wx_date(event.GetDate())
        self.shell_obj.date = new_date
        self.shell_obj.date_changed(new_date)

    #--------------------------------------------------------------------------
    # Private methods
    #--------------------------------------------------------------------------
    def set_date(self, date):
        """ Sets the date on the widget.

        """
        # wx will not fire an EVT_DATE_CHANGED when setting the
        # value, so we do not need any feeback guards here.
        self.widget.SetValue(to_wx_date(date))

    def set_min_date(self, date):
        """ Sets the minimum date on the widget with the provided value.

        """
        widget = self.widget
        widget.SetRange(to_wx_date(date), widget.GetUpperLimit())

    def set_max_date(self, date):
        """ Sets the maximum date on the widget with the provided value.

        """
        widget = self.widget
        widget.SetRange(widget.GetLowerLimit(), to_wx_date(date))

    def set_format(self, date_format):
        """ Sets the display format on the widget with the provided value.

        .. note:: Changing the format on wx is not supported.
                  See http://trac.wxwidgets.org/ticket/10988

        """
        pass

